@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King.
+ * Copyright (C) 2021-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,7 +48,7 @@
 /* used for table of perf events to gather */
 typedef struct {
 	const unsigned int type;	/* perf types */
-	unsigned long config;	/* perf type specific config */
+	unsigned long config;		/* perf type specific config */
 	const char *path;		/* perf trace point path (only for trace points) */
 	const char *label;		/* human readable name for perf type */
 } stress_perf_info_t;
@@ -61,9 +61,9 @@ typedef struct {
 } stress_perf_data_t;
 
 typedef struct {
-	const double	threshold;
-	const double	scale;
-	const char 	*suffix;
+	const double	threshold;	/* scaling threshold */
+	const double	scale;		/* scaling value */
+	const char 	*suffix;	/* scaling suffix, GB, MB, etc */
 } stress_perf_scale_t;
 
 /* Tracepoint */
@@ -261,6 +261,9 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("kmem/mm_page_alloc",		"MM Page Alloc"),
 	PERF_INFO_TP("kmem/mm_page_free",		"MM Page Free"),
 
+	PERF_INFO_TP("mmap_lock/mmap_lock_start_locking",	"MMAP lock start"),
+	PERF_INFO_TP("mmap_lock/mmap_lock_released",	"MMAP lock release"),
+
 	PERF_INFO_TP("rcu/rcu_utilization",		"RCU Utilization"),
 
 	PERF_INFO_TP("sched/sched_migrate_task",	"Sched Migrate Task"),
@@ -283,6 +286,11 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("irq/softirq_exit",		"Soft IRQ Exit"),
 	PERF_INFO_TP("nmi/nmi_handler",			"NMI handler"),
 
+	PERF_INFO_TP("block/block_bio_complete",	"Block BIO complete"),
+
+	PERF_INFO_TP("io_uring/io_uring_submit_sqe",	"IO uring submit"),
+	PERF_INFO_TP("io_uring/io_uring_complete",	"IO uring complete"),
+
 	PERF_INFO_TP("writeback/writeback_dirty_inode",	"Writeback Dirty Inode"),
 	PERF_INFO_TP("writeback/writeback_dirty_page",	"Writeback Dirty Page"),
 
@@ -291,6 +299,9 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("skb/consume_skb",			"SKB Consume"),
 	PERF_INFO_TP("skb/kfree_skb",			"SKB Kfree"),
 
+	PERF_INFO_TP("lock/contention_begin",		"Lock Contention Begin"),
+	PERF_INFO_TP("lock/contention_end",		"Lock Contention End"),
+
 	PERF_INFO_TP("iommu/io_page_fault",		"IOMMU IO Page Fault"),
 	PERF_INFO_TP("iommu/map",			"IOMMU Map"),
 	PERF_INFO_TP("iommu/unmap",			"IOMMU Unmap"),
@@ -306,6 +317,10 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	{ 0, 0, NULL, NULL }
 };
 
+/*
+ *  stress_perf_type_tracepoint_resolve_config()
+ *	resolve tracing event config value
+ */
 static inline void stress_perf_type_tracepoint_resolve_config(stress_perf_info_t *pi)
 {
 	char path[PATH_MAX];
@@ -328,6 +343,10 @@ static inline void stress_perf_type_tracepoint_resolve_config(stress_perf_info_t
 	pi->config = config;
 }
 
+/*
+ *  stress_perf_init()
+ *	perf initialize, resolve all configs
+ */
 void stress_perf_init(void)
 {
 	size_t i;
@@ -339,6 +358,10 @@ void stress_perf_init(void)
 	}
 }
 
+/*
+ *  stress_sys_perf_event_open()
+ *	perf_event_open syscall wrapper
+ */
 static inline int stress_sys_perf_event_open(
 	struct perf_event_attr *attr,
 	pid_t pid,
@@ -423,8 +446,7 @@ int stress_perf_open(stress_perf_t *sp)
 
 		ret = stress_lock_acquire(g_shared->perf.lock);
 		if (ret) {
-			pr_dbg("perf: lock on perf lock failed: %d (%s)\n",
-				ret, strerror(ret));
+			pr_dbg("perf: lock on perf lock failed\n");
 			return -1;
 		}
 		if (!g_shared->perf.no_perf) {
@@ -434,8 +456,7 @@ int stress_perf_open(stress_perf_t *sp)
 		}
 		ret = stress_lock_release(g_shared->perf.lock);
 		if (ret) {
-			pr_dbg("perf: unlock on perf lock failed: %d (%s)\n",
-				ret, strerror(ret));
+			pr_dbg("perf: unlock on perf lock failed\n");
 			return -1;
 		}
 		return -1;
@@ -458,7 +479,7 @@ int stress_perf_enable(stress_perf_t *sp)
 		return 0;
 
 	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
-		int fd = sp->perf_stat[i].fd;
+		const int fd = sp->perf_stat[i].fd;
 
 		if (fd > -1) {
 			if (ioctl(fd, PERF_EVENT_IOC_RESET,
@@ -491,7 +512,7 @@ int stress_perf_disable(stress_perf_t *sp)
 		return 0;
 
 	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
-		int fd = sp->perf_stat[i].fd;
+		const int fd = sp->perf_stat[i].fd;
 
 		if (fd > -1) {
 			if (ioctl(fd, PERF_EVENT_IOC_DISABLE,
@@ -521,7 +542,8 @@ int stress_perf_close(stress_perf_t *sp)
 		goto out_ok;
 
 	for (i = 0; i < STRESS_PERF_MAX && perf_info[i].label; i++) {
-		int fd = sp->perf_stat[i].fd;
+		const int fd = sp->perf_stat[i].fd;
+
 		if (fd < 0 ) {
 			sp->perf_stat[i].counter = STRESS_PERF_INVALID;
 			continue;
@@ -592,6 +614,10 @@ static const char *stress_perf_stat_scale(const uint64_t counter, const double d
 	return buffer;
 }
 
+/*
+ *  stress_perf_stat_dump()
+ *	emit perf statistics
+ */
 void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const double duration)
 {
 	bool no_perf_stats = true;
@@ -654,7 +680,7 @@ void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const
 
 		for (p = 0; p < STRESS_PERF_MAX && perf_info[p].label; p++) {
 			const char *l = perf_info[p].label;
-			uint64_t ct = counter_totals[p];
+			const uint64_t ct = counter_totals[p];
 
 			if (l && (ct != STRESS_PERF_INVALID)) {
 				char extra[32];
@@ -702,14 +728,14 @@ void stress_perf_stat_dump(FILE *yaml, stress_stressor_t *stressors_list, const
 			ssize_t ret;
 			bool paranoid = false;
 			int level = 0;
-			static char *path = "/proc/sys/kernel/perf_event_paranoid";
+			static const char *path = "/proc/sys/kernel/perf_event_paranoid";
 
 			ret = system_read(path, buffer, sizeof(buffer) - 1);
 			if (ret > 0) {
 				if (sscanf(buffer, "%5d", &level) == 1)
 					paranoid = true;
 			}
-			if (paranoid & (level > 1)) {
+			if (paranoid && (level > 1)) {
 				pr_inf("Cannot read perf counters, "
 					"do not have CAP_SYS_ADMIN capability "
 					"or %s is set too high (%d)\n",
