@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +31,12 @@
 #define HAVE_PKEY_SET
 #endif
 
+#if defined(HAVE_GRP_H)
+#include <grp.h>
+#else
+UNEXPECTED
+#endif
+
 #if defined(HAVE_USTAT_H)
 #if defined(__sun__)
 /* ustat and long file support on sun does not build */
@@ -52,8 +58,8 @@
 #include <asm/ldt.h>
 #endif
 
-#if defined(HAVE_SYS_MOUNT_H)
-#include <sys/mount.h>
+#if defined(HAVE_SYS_PIDFD_H)
+#include <sys/pidfd.h>
 #endif
 
 #if defined(HAVE_SYS_PRCTL_H)
@@ -70,6 +76,7 @@
 #elif defined(HAVE_ATTR_XATTR_H)
 #include <attr/xattr.h>
 #endif
+
 /*  Sanity check */
 #if defined(HAVE_SYS_XATTR_H) &&        \
     defined(HAVE_ATTR_XATTR_H)
@@ -82,6 +89,17 @@
 #define shim_rusage_who_t	int
 #endif
 
+#if defined(__sun__)
+#if defined(HAVE_GETDOMAINNAME)
+extern int getdomainname(char *name, size_t len);
+#endif
+#if defined(HAVE_SETDOMAINNAME)
+extern int setdomainname(const char *name, size_t len);
+#endif
+#endif
+
+#define FALLOCATE_BUF_SIZE	(8192)
+
 /*
  *  Various shim abstraction wrappers around systems calls and
  *  GCC helper functions that may not be supported by some
@@ -176,20 +194,19 @@ ssize_t shim_copy_file_range(
  */
 static int shim_emulate_fallocate(int fd, off_t offset, off_t len)
 {
-	const off_t buf_sz = 8192;
-	char buffer[buf_sz];
+	char buffer[FALLOCATE_BUF_SIZE];
 	off_t n;
 
 	n = lseek(fd, offset, SEEK_SET);
 	if (n == (off_t)-1)
 		return -1;
 
-	(void)memset(buffer, 0, buf_sz);
+	(void)memset(buffer, 0, FALLOCATE_BUF_SIZE);
 	n = len;
 
 	while (keep_stressing_flag() && (n > 0)) {
 		ssize_t ret;
-		size_t count = (size_t)STRESS_MINIMUM(n, buf_sz);
+		const size_t count = (size_t)STRESS_MINIMUM(n, FALLOCATE_BUF_SIZE);
 
 		ret = write(fd, buffer, count);
 		if (ret >= 0) {
@@ -767,11 +784,11 @@ int shim_nanosleep_uint64(uint64_t nsec)
 		errno = 0;
 		if (usleep(usec) < 0) {
 			if (errno == EINTR) {
-				double t_left = t_end - stress_time_now();
+				const double t_left = t_end - stress_time_now();
 
 				if (t_left < 0.0)
 					return 0;
-				usec = (useconds_t)(t_left * 1000000.0);
+				usec = (useconds_t)(t_left * STRESS_DBL_MICROSECOND);
 				if (usec == 0)
 					return 0;
 				if (keep_stressing_flag())
@@ -1138,13 +1155,10 @@ int shim_brk(void *addr)
 #elif defined(HAVE_BRK)
 	return brk(addr);
 #else
-	uintptr_t brkaddr;
-	intptr_t inc;
-	void *newbrk;
+	const uintptr_t brkaddr = (uintptr_t)shim_sbrk(0);
+	intptr_t inc = brkaddr - (intptr_t)addr;
+	const void *newbrk = shim_sbrk(inc);
 
-	brkaddr = (uintptr_t)shim_sbrk(0);
-	inc = brkaddr - (intptr_t)addr;
-	newbrk = shim_sbrk(inc);
 	if (newbrk == (void *)-1) {
 		if (errno != ENOSYS)
 			errno = ENOMEM;
@@ -1230,16 +1244,14 @@ size_t shim_strlcat(char *dst, const char *src, size_t len)
 	register const char *s = src;
 	register size_t n = len, tmplen;
 
-	while (n-- && *d != '\0') {
+	while (n-- && *d != '\0')
 		d++;
-	}
 
 	tmplen = d - dst;
 	n = len - tmplen;
 
-	if (!n) {
+	if (!n)
 		return strlen(s) + tmplen;
-	}
 
 	while (*s != '\0') {
 		if (n != 1) {
@@ -1600,10 +1612,10 @@ int shim_move_mount(
 }
 
 /*
- *  sys_clone3()
+ *  shim_clone3()
  *	Linux clone3 system call wrapper
  */
-int sys_clone3(struct shim_clone_args *cl_args, size_t size)
+int shim_clone3(struct shim_clone_args *cl_args, size_t size)
 {
 #if defined(__NR_clone3) &&	\
     defined(HAVE_SYSCALL)
@@ -1917,6 +1929,22 @@ int shim_clock_getres(clockid_t clk_id, struct timespec *res)
 #endif
 }
 
+/*
+ *   shim_clock_adjtime
+ *	wrapper for linux clock_adjtime system call
+ */
+int shim_clock_adjtime(clockid_t clk_id, shim_timex_t *buf)
+{
+#if defined(CLOCK_THREAD_CPUTIME_ID) && \
+    defined(HAVE_CLOCK_ADJTIME) &&	\
+    defined(HAVE_SYS_TIMEX_H) &&	\
+    defined(HAVE_TIMEX)
+	return clock_adjtime(clk_id, buf);
+#else
+	return (int)shim_enosys(0, clk_id, buf);
+#endif
+}
+
 /*
  *   shim_clock_gettime
  *	wrapper for linux clock_gettime system call,
@@ -1973,7 +2001,14 @@ int shim_nice(int inc)
 {
 #if defined(__NR_nice) &&	\
     defined(HAVE_SYSCALL)
-	return (int)syscall(__NR_nice, inc);
+	int ret;
+
+	ret = (int)syscall(__NR_nice, inc);
+	if ((ret < 0) && (errno == ENOSYS)) {
+		errno = 0;
+		return nice(inc);
+	}
+	return ret;
 #elif defined(HAVE_NICE)
 	return nice(inc);
 #else
@@ -2140,15 +2175,15 @@ int shim_vhangup(void)
  */
 int shim_arch_prctl(int code, unsigned long addr)
 {
-#if defined(HAVE_ARCH_PRCTL) &&		\
-    defined(__linux__)
-        extern int arch_prctl();
-
-        return arch_prctl(code, addr);
-#elif defined(__NR_arch_prctl) && 	\
+#if defined(__NR_arch_prctl) && 	\
       defined(__linux__) &&		\
       defined(HAVE_SYSCALL)
         return (int)syscall(__NR_arch_prctl, code, addr);
+#elif defined(HAVE_ARCH_PRCTL) &&	\
+       defined(__linux__)
+        extern int arch_prctl();
+
+        return arch_prctl(code, addr);
 #else
         return (int)shim_enosys(0, code, addr);
 #endif
@@ -2337,3 +2372,54 @@ int shim_force_rmdir(const char *pathname)
 {
 	return rmdir(pathname);
 }
+
+/*
+ *  shim_getdomainname()
+ *	shim required for sun due to non-extern
+ */
+int shim_getdomainname(char *name, size_t len)
+{
+#if defined(HAVE_GETDOMAINNAME)
+	return getdomainname(name, len);
+#elif defined(__NR_getdomainname) &&	\
+      defined(HAVE_SYSCALL)
+	return (int)syscall(__NR_getdomainname, name, len);
+#else
+	return (int)shim_enosys(0, name, len);
+#endif
+}
+
+/*
+ *  shim_setdomainname()
+ *	shim required for sun due to non-extern
+ */
+int shim_setdomainname(const char *name, size_t len)
+{
+#if defined(HAVE_GETDOMAINNAME)
+	return setdomainname(name, len);
+#elif defined(__NR_setdomainname) &&	\
+      defined(HAVE_SYSCALL)
+	return (int)syscall(__NR_setdomainname, name, len);
+#else
+	return (int)shim_enosys(0, name, len);
+#endif
+}
+
+/*
+ *  shim_setgroups()
+ *	shim for POSIX version of setgroups
+ */
+int shim_setgroups(int size, const gid_t *list)
+{
+#if defined(HAVE_GRP_H)
+#if defined(__linux__)
+	/* Linux passes size as size_t */
+	return setgroups((size_t)size, list);
+#else
+	/* POSIX variant, as used by BSD etc */
+	return setgroups(size, list);
+#endif
+#else
+	return (int)shim_enosys(size, list);
+#endif
+}
