@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2021 Canonical, Ltd.
+ * Copyright (C)      2021 Canonical, Ltd.
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +16,144 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-pragma.h"
+#include "core-thermal-zone.h"
+
+#if defined(HAVE_MACH_MACH_H)
+#include <mach/mach.h>
+#endif
+
+#if defined(HAVE_MACH_MACHINE_H)
+#include <mach/machine.h>
+#endif
+
+#if defined(HAVE_MACH_VM_STATISTICS_H)
+#include <mach/vm_statistics.h>
+#endif
+
+#if defined(HAVE_SYS_SYSCTL_H)
+STRESS_PRAGMA_PUSH
+STRESS_PRAGMA_WARN_CPP_OFF
+#include <sys/sysctl.h>
+STRESS_PRAGMA_POP
+#endif
+
+#include <float.h>
+
+#if defined(HAVE_SYS_SYSMACROS_H)
+#include <sys/sysmacros.h>
+#endif
+
+#if defined(HAVE_SYS_VMMETER_H)
+#include <sys/vmmeter.h>
+#endif
+
+#if defined(HAVE_UVM_UVM_EXTERN_H)
+#include <uvm/uvm_extern.h>
+#endif
+
+#if defined(HAVE_MNTENT_H)
+#include <mntent.h>
+#endif
+
+/* vmstat information */
+typedef struct {			/* vmstat column */
+	uint64_t	procs_running;	/* r */
+	uint64_t	procs_blocked;	/* b */
+	uint64_t	swap_total;	/* swpd info, total */
+	uint64_t	swap_free;	/* swpd info, free */
+	uint64_t	swap_used;	/* swpd used = total - free */
+	uint64_t	memory_free;	/* free */
+	uint64_t	memory_buff;	/* buff */
+	uint64_t	memory_cached;	/* cached */
+	uint64_t	memory_reclaimable; /* reclaimabled cached */
+	uint64_t	swap_in;	/* si */
+	uint64_t	swap_out;	/* so */
+	uint64_t	block_in;	/* bi */
+	uint64_t	block_out;	/* bo */
+	uint64_t	interrupt;	/* in */
+	uint64_t	context_switch;	/* cs */
+	uint64_t	user_time;	/* us */
+	uint64_t	system_time;	/* sy */
+	uint64_t	idle_time;	/* id */
+	uint64_t	wait_time;	/* wa */
+	uint64_t	stolen_time;	/* st */
+} stress_vmstat_t;
+
+/* iostat information, from /sys/block/$dev/stat */
+typedef struct {
+	uint64_t	read_io;	/* number of read I/Os processed */
+	uint64_t	read_merges;	/* number of read I/Os merged with in-queue I/O */
+	uint64_t	read_sectors;	/* number of sectors read */
+	uint64_t	read_ticks;	/* total wait time for read requests */
+	uint64_t	write_io;	/* number of write I/Os processed */
+	uint64_t	write_merges;	/* number of write I/Os merged with in-queue I/O */
+	uint64_t	write_sectors;	/* number of sectors written */
+	uint64_t	write_ticks;	/* total wait time for write requests */
+	uint64_t	in_flight;	/* number of I/Os currently in flight */
+	uint64_t	io_ticks;	/* total time this block device has been active */
+	uint64_t	time_in_queue;	/* total wait time for all requests */
+	uint64_t	discard_io;	/* number of discard I/Os processed */
+	uint64_t	discard_merges;	/* number of discard I/Os merged with in-queue I/O */
+	uint64_t	discard_sectors;/* number of sectors discarded */
+	uint64_t	discard_ticks;	/* total wait time for discard requests */
+} stress_iostat_t;
 
 static int32_t vmstat_delay = 0;
 static int32_t thermalstat_delay = 0;
 static int32_t iostat_delay = 0;
 
+#if defined(__FreeBSD__)
+static void freebsd_get_cpu_time(
+	uint64_t *user_time,
+	uint64_t *system_time,
+	uint64_t *idle_time)
+{
+	const int cpus = stress_bsd_getsysctl_int("kern.smp.cpus");
+	long int *vals;
+	int i;
+
+	*user_time = 0;
+	*system_time = 0;
+	*idle_time = 0;
+
+	vals = (long int *)calloc(cpus * 5, sizeof(*vals));
+	if (!vals)
+		return;
+
+	if (stress_bsd_getsysctl("kern.cp_times", vals, cpus * 5 * sizeof(*vals)) < 0)
+		return;
+	for (i = 0; i < cpus * 5; i += 5) {
+		*user_time += vals[i];
+		*system_time += vals[i + 2];
+		*idle_time += vals[i + 4];
+	}
+	free(vals);
+}
+#endif
+
+#if defined(__NetBSD__)
+static void netbsd_get_cpu_time(
+	uint64_t *user_time,
+	uint64_t *system_time,
+	uint64_t *idle_time)
+{
+	long int vals[5];
+
+	*user_time = 0;
+	*system_time = 0;
+	*idle_time = 0;
+
+	if (stress_bsd_getsysctl("kern.cp_time", vals, sizeof(vals)) < 0)
+		return;
+	*user_time = vals[0];
+	*system_time = vals[2];
+	*idle_time = vals[4];;
+}
+#endif
+
 static int stress_set_generic_stat(
 	const char *const opt,
 	const char *name,
@@ -48,6 +174,7 @@ int stress_set_vmstat(const char *const opt)
 
 int stress_set_thermalstat(const char *const opt)
 {
+	g_opt_flags |= OPT_FLAGS_TZ_INFO;
 	return stress_set_generic_stat(opt, "thermalstat", &thermalstat_delay);
 }
 
@@ -56,124 +183,110 @@ int stress_set_iostat(const char *const opt)
 	return stress_set_generic_stat(opt, "iostat", &iostat_delay);
 }
 
-#if defined(__linux__)
+/*
+ *  stress_find_mount_dev()
+ *	find the path of the device that the file is located on
+ */
+char *stress_find_mount_dev(const char *name)
+{
+#if defined(__linux__) && 	\
+    defined(HAVE_GETMNTENT) &&	\
+    defined(HAVE_ENDMNTENT) &&	\
+    defined(HAVE_SETMNTENT)
+	static char dev_path[PATH_MAX];
+	struct stat statbuf;
+	dev_t dev;
+	FILE *mtab_fp;
+	struct mntent *mnt;
+
+	if (stat(name, &statbuf) < 0)
+		return NULL;
+
+	/* Cater for UBI char mounts */
+	if (S_ISBLK(statbuf.st_mode) || S_ISCHR(statbuf.st_mode))
+		dev = statbuf.st_rdev;
+	else
+		dev = statbuf.st_dev;
+
+	/* Try /proc/mounts then /etc/mtab */
+	mtab_fp = setmntent("/proc/mounts", "r");
+	if (!mtab_fp) {
+		mtab_fp = setmntent("/etc/mtab", "r");
+		if (!mtab_fp)
+			return NULL;
+	}
+
+	while ((mnt = getmntent(mtab_fp))) {
+		if ((!strcmp(name, mnt->mnt_dir)) ||
+		    (!strcmp(name, mnt->mnt_fsname)))
+			break;
+
+		if ((mnt->mnt_fsname[0] == '/') &&
+		    (stat(mnt->mnt_fsname, &statbuf) == 0) &&
+		    (statbuf.st_rdev == dev))
+			break;
+
+		if ((stat(mnt->mnt_dir, &statbuf) == 0) &&
+		    (statbuf.st_dev == dev))
+			break;
+	}
+	(void)endmntent(mtab_fp);
+
+	if (!mnt)
+		return NULL;
+	if (!mnt->mnt_fsname)
+		return NULL;
+	return realpath(mnt->mnt_fsname, dev_path);
+#elif defined(HAVE_SYS_SYSMACROS_H)
+	static char dev_path[PATH_MAX];
+	struct stat statbuf;
+	dev_t dev;
+	DIR *dir;
+	struct dirent *d;
+	dev_t majdev;
+
+	if (stat(name, &statbuf) < 0)
+		return NULL;
+
+	/* Cater for UBI char mounts */
+	if (S_ISBLK(statbuf.st_mode) || S_ISCHR(statbuf.st_mode))
+		dev = statbuf.st_rdev;
+	else
+		dev = statbuf.st_dev;
+
+	majdev = makedev(major(dev), 0);
+
+	dir = opendir("/dev");
+	if (!dir)
+		return NULL;
+
+	while ((d = readdir(dir)) != NULL) {
+		int ret;
+		struct stat stat_buf;
+
+		stress_mk_filename(dev_path, sizeof(dev_path), "/dev", d->d_name);
+		ret = stat(dev_path, &stat_buf);
+		if ((ret == 0) &&
+		    (S_ISBLK(stat_buf.st_mode)) &&
+		    (stat_buf.st_rdev == majdev)) {
+			(void)closedir(dir);
+			return dev_path;
+		}
+	}
+	(void)closedir(dir);
+
+	return NULL;
+#else
+	(void)name;
+
+	return NULL;
+#endif
+}
 
 static pid_t vmstat_pid;
 
-/*
- *  stress_iostat_follow_link
- *	recursively follow link until no more links
- */
-static void stress_iostat_follow_link(
-	const char *path,
-	char *buf,
-	const size_t buflen)
-{
-	char tmp[buflen], prev[buflen];
-
-	(void)memset(prev, 0, sizeof(prev));
-
-	shim_strlcpy(tmp, path, buflen);
-	for (;;) {
-		ssize_t ret;
-
-		ret = shim_readlink(tmp, buf, buflen - 1);
-		if (ret < 0) {
-			(void)shim_strlcpy(buf, path, buflen);
-			return;
-		}
-		buf[ret] = '\0';
-		if (!strncmp(prev, buf, buflen))
-			return;
-		(void)shim_strlcpy(prev, buf, buflen);
-	}
-}
-
-/*
- *  stress_iostat_dev_trim()
- *	trim device numbers off end, /dev/sda2 -> /dev/sda
- */
-static void stress_iostat_dev_trim(char *devname)
-{
-	char *ptr;
-
-	for (ptr = devname; *ptr; ptr++)
-		;
-
-	for (--ptr; (ptr > devname) && isdigit((int)*ptr); ptr--)
-		*ptr = '\0';
-}
-
-/*
- *  stress_iostat_dev_trim_nvme()
- *	trim 'p' character off end for an already trimmed nvme devname,
- *	/dev/nvme0n1p -> /dev/nvme0n1
- */
-static void stress_iostat_dev_trim_nvme(char *devname)
-{
-	char *ptr;
-
-	for (ptr = devname; *ptr; ptr++)
-		;
-
-	if ((--ptr > devname) && (*ptr == 'p'))
-		*ptr = '\0';
-}
-
-/*
- *  stress_iostat_get_iostat_name
- *	try to find the /sys/block/$dev/stat name from a given
- *	device number. Returns null if not found.
- */
-static char *stress_iostat_get_iostat_name(
-	const dev_t rdev,
-	char *name,
-	const size_t namelen)
-{
-	DIR *dp;
-	struct dirent *d;
-	struct stat statbuf;
-
-	(void)memset(name, 0, namelen);
-
-	dp = opendir("/dev/");
-	if (!dp)
-		return NULL;
-	while ((d = readdir(dp)) != NULL) {
-		if ((d->d_type & DT_BLK) == 0)
-			continue;
-
-		(void)snprintf(name, namelen, "/dev/%s", d->d_name);
-		if (stat(name, &statbuf) < 0)
-			continue;
-		if (statbuf.st_rdev == rdev) {
-			(void)closedir(dp);
-
-			(void)snprintf(name, namelen, "/sys/block/%s/stat", d->d_name);
-			if (stat(name, &statbuf) == 0)
-				return name;
-
-			/* strip off digits from end of dev, retry */
-			stress_iostat_dev_trim(d->d_name);
-			(void)snprintf(name, namelen, "/sys/block/%s/stat", d->d_name);
-			if (stat(name, &statbuf) == 0)
-				return name;
-
-			/* strip off 'p' from end of dev in case of a nvme
-			 * device, retry */
-			stress_iostat_dev_trim_nvme(d->d_name);
-			(void)snprintf(name, namelen, "/sys/block/%s/stat", d->d_name);
-			if (stat(name, &statbuf) == 0)
-				return name;
-
-			(void)memset(name, 0, namelen);
-			return NULL;
-		}
-	}
-	(void)memset(name, 0, namelen);
-	(void)closedir(dp);
-	return NULL;
-}
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 
 /*
  *  stress_iostat_iostat_name()
@@ -184,102 +297,51 @@ static char *stress_iostat_iostat_name(
 	char *iostat_name,
 	const size_t iostat_name_len)
 {
-	const char *temp_path = stress_get_temp_path();
+	char *temp_path, *dev, *ptr;
 	struct stat statbuf;
-	int ret;
-	FILE *fp;
-	char buf[4096];
-	int f_major, f_minor;
 
 	/* Resolve links */
-	stress_iostat_follow_link(temp_path, iostat_name, iostat_name_len);
-
-	/* Get dev info */
-	ret = stat(iostat_name, &statbuf);
-	if (ret < 0) {
-		*iostat_name = '\0';
+	temp_path = realpath(stress_get_temp_path(), NULL);
+	if (!temp_path)
 		return NULL;
-	}
+
+	/* Find device */
+	dev = stress_find_mount_dev(temp_path);
+	if (!dev)
+		return NULL;
+
+	/* Skip over leading /dev */
+	if (!strncmp(dev, "/dev", 4))
+		dev += 4;
+	if (*dev == '/')
+		dev++;
+
+	ptr = dev + strlen(dev) - 1;
 
 	/*
-	 *  Scan mountinfo for mount points that match the device
+	 *  Try /dev/sda12, then /dev/sda1, then /dev/sda, then terminate
 	 */
-	fp = fopen("/proc/self/mountinfo", "r");
-	if (!fp) {
-		*iostat_name = '\0';
-		return NULL;
+	while (ptr >= dev) {
+		(void)snprintf(iostat_name, iostat_name_len, "/sys/block/%s/stat", dev);
+		if (stat(iostat_name, &statbuf) == 0)
+			return iostat_name;
+		if (!isdigit(*ptr))
+			break;
+		*ptr = '\0';
+		ptr--;
 	}
 
-	f_major = major(statbuf.st_dev);
-	f_minor = major(statbuf.st_dev);
-
-	(void)memset(buf, 0, sizeof(buf));
-	while (fgets(buf, sizeof(buf), fp)) {
-		int mnt_major, mnt_minor;
-
-		ret = sscanf(buf, "%*d %*d %d:%d", &mnt_major, &mnt_minor);
-		if (ret != 2)
-			continue;
-		if ((f_major == mnt_major) && (f_minor == mnt_minor)) {
-			/*
-			 * parse something like:
-			 *  31 1 253:1 / / rw,relatime shared:1 - ext4 /dev/mapper/vgubuntu-root rw,errors=remount-ro
-			 *  ..and look for - field, e.g. ext4 /dev/mapper/vgubuntu-root
-			 */
-			char *start = strstr(buf, " - "), *end, *tmp;
-
-			if (!start)
-				continue;
-
-			/* Skip over ' - ' and following non-spaces */
-			for (start += 3; *start && *start != ' '; start++)
-				start++;
-			if (!*start)
-				continue;
-
-			/* skip over spaces to get to dev name */
-			while (*start == ' ')
-				start++;
-			if (!*start)
-				continue;
-
-			/* now look for the end of dev name */
-			for (end = start; *end && *end != ' '; end++)
-				;
-			if (!*end)
-				continue;
-			*end = '\0';
-
-			/* get rdev block info of this device */
-			ret = stat(start, &statbuf);
-			if (ret < 0)
-				continue;
-
-			/* ..and find any matching devices in /dev and resolve the iostate_name */
-			tmp = stress_iostat_get_iostat_name(statbuf.st_rdev, iostat_name, iostat_name_len);
-			if (tmp) {
-				(void)fclose(fp);
-				return tmp;
-			}
-		}
-	}
-	/* nothing found */
-	(void)fclose(fp);
-	*iostat_name = '\0';
-
 	return NULL;
 }
 
 /*
  *  stress_read_iostat()
- *	read the stats from an iostat stat file
+ *	read the stats from an iostat stat file, linux variant
  */
 static void stress_read_iostat(const char *iostat_name, stress_iostat_t *iostat)
 {
 	FILE *fp;
 
-	(void)memset(iostat, 0, sizeof(*iostat));
-
 	fp = fopen(iostat_name, "r");
 	if (fp) {
 		int ret;
@@ -308,7 +370,6 @@ static void stress_read_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	}
 }
 
-
 #define STRESS_IOSTAT_DELTA(field)					\
 	iostat->field = ((iostat_current.field > iostat_prev.field) ?	\
 	(iostat_current.field - iostat_prev.field) : 0)
@@ -322,6 +383,7 @@ static void stress_get_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	static stress_iostat_t iostat_prev;
 	stress_iostat_t iostat_current;
 
+	(void)memset(&iostat_current, 0, sizeof(iostat_current));
 	stress_read_iostat(iostat_name, &iostat_current);
 	STRESS_IOSTAT_DELTA(read_io);
 	STRESS_IOSTAT_DELTA(read_merges);
@@ -340,7 +402,9 @@ static void stress_get_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	STRESS_IOSTAT_DELTA(discard_ticks);
 	(void)memcpy(&iostat_prev, &iostat_current, sizeof(iostat_prev));
 }
+#endif
 
+#if defined(__linux__)
 /*
  *  stress_next_field()
  *	skip to next field, returns false if end of
@@ -375,8 +439,6 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 	FILE *fp;
 	char buffer[1024];
 
-	(void)memset(vmstat, 0, sizeof(*vmstat));
-
 	fp = fopen("/proc/stat", "r");
 	if (fp) {
 		while (fgets(buffer, sizeof(buffer), fp)) {
@@ -453,13 +515,13 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 			if (!strncmp(buffer, "procs_running", 13)) {
 				if (!stress_next_field(&ptr))
 					continue;
-				/* context switches */
+				/* processes running */
 				vmstat->procs_running = (uint64_t)atoll(ptr);
 			}
 			if (!strncmp(buffer, "procs_blocked", 13)) {
 				if (!stress_next_field(&ptr))
 					continue;
-				/* context switches */
+				/* procesess blocked */
 				vmstat->procs_blocked = (uint64_t)atoll(ptr);
 			}
 			if (!strncmp(buffer, "swap", 4)) {
@@ -495,13 +557,23 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 			if (!strncmp(buffer, "Cached", 6)) {
 				if (!stress_next_field(&ptr))
 					continue;
-				vmstat->memory_cache = (uint64_t)atoll(ptr);
+				vmstat->memory_cached = (uint64_t)atoll(ptr);
+			}
+			if (!strncmp(buffer, "KReclaimable", 12)) {
+				if (!stress_next_field(&ptr))
+					continue;
+				vmstat->memory_reclaimable = (uint64_t)atoll(ptr);
 			}
 			if (!strncmp(buffer, "SwapTotal", 9)) {
 				if (!stress_next_field(&ptr))
 					continue;
 				vmstat->swap_total = (uint64_t)atoll(ptr);
 			}
+			if (!strncmp(buffer, "SwapFree", 8)) {
+				if (!stress_next_field(&ptr))
+					continue;
+				vmstat->swap_free = (uint64_t)atoll(ptr);
+			}
 			if (!strncmp(buffer, "SwapUsed", 8)) {
 				if (!stress_next_field(&ptr))
 					continue;
@@ -509,6 +581,12 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 			}
 		}
 		(void)fclose(fp);
+
+		if ((vmstat->swap_used == 0) &&
+		    (vmstat->swap_free > 0) &&
+		    (vmstat->swap_total > 0)) {
+			vmstat->swap_used = vmstat->swap_total - vmstat->swap_free;
+		}
 	}
 
 	fp = fopen("/proc/vmstat", "r");
@@ -540,6 +618,185 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 		(void)fclose(fp);
 	}
 }
+#elif defined(__FreeBSD__)
+/*
+ *  stress_read_vmstat()
+ *	read vmstat statistics, FreeBSD variant, partially implemented
+ */
+static void stress_read_vmstat(stress_vmstat_t *vmstat)
+{
+#if defined(HAVE_SYS_VMMETER_H)
+	struct vmtotal t;
+#endif
+
+	vmstat->interrupt = stress_bsd_getsysctl_uint64("vm.stats.sys.v_intr");
+	vmstat->context_switch = stress_bsd_getsysctl_uint64("vm.stats.sys.v_swtch");
+	vmstat->swap_in = stress_bsd_getsysctl_uint64("vm.stats.vm.v_swapin");
+	vmstat->swap_out = stress_bsd_getsysctl_uint64("vm.stats.vm.v_swapout");
+	vmstat->block_in = stress_bsd_getsysctl_uint64("vm.stats.vm.v_vnodepgsin");
+	vmstat->block_out = stress_bsd_getsysctl_uint64("vm.stats.vm.v_vnodepgsin");
+	vmstat->memory_free = (uint64_t)stress_bsd_getsysctl_uint32("vm.stats.vm.v_free_count");
+	vmstat->memory_cached = (uint64_t)stress_bsd_getsysctl_uint("vm.stats.vm.v_cache_count");
+
+	freebsd_get_cpu_time(&vmstat->user_time, &vmstat->system_time, &vmstat->idle_time);
+
+#if defined(HAVE_SYS_VMMETER_H)
+	if (stress_bsd_getsysctl("vm.vmtotal", &t, sizeof(t)) == 0) {
+		vmstat->procs_running = t.t_rq - 1;
+		vmstat->procs_blocked = t.t_dw + t.t_pw;
+	}
+#endif
+}
+#elif defined(__NetBSD__)
+/*
+ *  stress_read_vmstat()
+ *	read vmstat statistics, NetBSD variant, partially implemented
+ */
+static void stress_read_vmstat(stress_vmstat_t *vmstat)
+{
+#if defined(HAVE_SYS_VMMETER_H)
+	struct vmtotal t;
+#endif
+#if defined(HAVE_UVM_UVM_EXTERN_H)
+	struct uvmexp_sysctl u;
+#endif
+	netbsd_get_cpu_time(&vmstat->user_time, &vmstat->system_time, &vmstat->idle_time);
+#if defined(HAVE_UVM_UVM_EXTERN_H)
+	if (stress_bsd_getsysctl("vm.uvmexp2", &u, sizeof(u)) == 0) {
+		vmstat->memory_cached = u.filepages;	/* Guess */
+		vmstat->interrupt = u.intrs;
+		vmstat->context_switch = u.swtch;
+		vmstat->swap_in = u.pgswapin;
+		vmstat->swap_out = u.pgswapout;
+		vmstat->swap_used = u.swpginuse;
+		vmstat->memory_free = u.free;
+	}
+#endif
+
+#if defined(HAVE_SYS_VMMETER_H)
+	if (stress_bsd_getsysctl("vm.vmmeter", &t, sizeof(t)) == 0) {
+		vmstat->procs_running = t.t_rq - 1;
+		vmstat->procs_blocked = t.t_dw + t.t_pw;
+	}
+#endif
+}
+#elif defined(__APPLE__) &&		\
+      defined(HAVE_SYS_SYSCTL_H) &&	\
+      defined(HAVE_MACH_MACH_H) &&	\
+      defined(HAVE_MACH_VM_STATISTICS_H)
+/*
+ *  stress_read_vmstat()
+ *	read vmstat statistics, OS X variant, partially implemented
+ */
+static void stress_read_vmstat(stress_vmstat_t *vmstat)
+{
+	vm_statistics64_data_t vm_stat;
+	struct xsw_usage xsu;
+	mach_port_t host = mach_host_self();
+	natural_t count = HOST_VM_INFO64_COUNT;
+	natural_t pcount, pi_array_count;
+	processor_info_array_t pi_array;
+	size_t page_size = stress_get_page_size();
+	int ret;
+
+	ret = host_statistics64(host, HOST_VM_INFO64, (host_info64_t)&vm_stat, &count);
+	if (ret >= 0) {
+		vmstat->swap_in = vm_stat.pageins;
+		vmstat->swap_out = vm_stat.pageouts;
+		vmstat->memory_free = (page_size / 1024) * vm_stat.free_count;
+	}
+	ret = stress_bsd_getsysctl("vm.swapusage", &xsu, sizeof(xsu));
+	if (ret >= 0) {
+		vmstat->swap_total = xsu.xsu_total;
+		vmstat->swap_used = xsu.xsu_used;
+		vmstat->swap_free = xsu.xsu_avail;
+	}
+	vmstat->user_time = 0;
+	vmstat->system_time= 0;
+	vmstat->idle_time = 0;
+	vmstat->wait_time = 0;
+	vmstat->stolen_time = 0;
+#if defined(HAVE_MACH_MACH_H) &&	\
+    defined(PROCESSOR_CPU_LOAD_INFO) && \
+    defined(CPU_STATE_USER) &&		\
+    defined(CPU_STATE_SYSTEM) &&	\
+    defined(CPU_STATE_IDLE)
+	ret = host_processor_info(host, PROCESSOR_CPU_LOAD_INFO, &pcount, &pi_array, &pi_array_count);
+	if (ret >= 0) {
+		natural_t i;
+
+
+		for (i = 0; i < pi_array_count; i++) {
+			integer_t *cpu_ticks = &pi_array[i * CPU_STATE_MAX];
+
+			vmstat->user_time += cpu_ticks[CPU_STATE_USER];
+			vmstat->system_time += cpu_ticks[CPU_STATE_SYSTEM];
+			vmstat->idle_time += cpu_ticks[CPU_STATE_IDLE];
+		}
+	}
+#endif
+#if defined(HAVE_SYS_SYSCTL_H) &&	\
+    defined(CTL_KERN) &&		\
+    defined(KERN_PROC) &&		\
+    defined(KERN_PROC_ALL) &&		\
+    defined(SRUN)
+	{
+		size_t length;
+		static const int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0 };
+
+		vmstat->procs_running = 0;
+		vmstat->procs_blocked = 0;
+
+		for (;;) {
+			struct kinfo_proc * result;
+			size_t i, n;
+
+			ret = sysctl((int *)name, (sizeof(name)/sizeof(*name))-1, NULL,
+				&length, NULL, 0);
+			if (ret < 0)
+				break;
+
+			result = malloc(length);
+			if (!result)
+				break;
+
+			ret = sysctl((int *)name, (sizeof(name)/sizeof(*name))-1, result,
+				&length, NULL, 0);
+			if (ret < 0) {
+				free(result);
+				break;
+			}
+
+			n = length / sizeof(struct kinfo_proc);
+			for (i = 0; i < n; i++) {
+				if (result[i].kp_proc.p_flag & P_SYSTEM)
+					continue;
+				switch (result[i].kp_proc.p_stat) {
+				case SRUN:
+					vmstat->procs_running++;
+					break;
+				default:
+					vmstat->procs_blocked++;
+					break;
+				}
+			}
+			free(result);
+			if (ret == 0)
+				break;
+		}
+	}
+#endif
+}
+#else
+/*
+ *  stress_read_vmstat()
+ *	read vmstat statistics, no-op
+ */
+static void stress_read_vmstat(stress_vmstat_t *vmstat)
+{
+	(void)vmstat;
+}
+#endif
 
 #define STRESS_VMSTAT_COPY(field)	vmstat->field = (vmstat_current.field)
 #define STRESS_VMSTAT_DELTA(field)					\
@@ -555,14 +812,18 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
 	static stress_vmstat_t vmstat_prev;
 	stress_vmstat_t vmstat_current;
 
+	(void)memset(&vmstat_current, 0, sizeof(vmstat_current));
+	(void)memset(vmstat, 0, sizeof(*vmstat));
 	stress_read_vmstat(&vmstat_current);
 	STRESS_VMSTAT_COPY(procs_running);
 	STRESS_VMSTAT_COPY(procs_blocked);
 	STRESS_VMSTAT_COPY(swap_total);
 	STRESS_VMSTAT_COPY(swap_used);
+	STRESS_VMSTAT_COPY(swap_free);
 	STRESS_VMSTAT_COPY(memory_free);
 	STRESS_VMSTAT_COPY(memory_buff);
-	STRESS_VMSTAT_COPY(memory_cache);
+	STRESS_VMSTAT_COPY(memory_cached);
+	STRESS_VMSTAT_COPY(memory_reclaimable);
 	STRESS_VMSTAT_DELTA(swap_in);
 	STRESS_VMSTAT_DELTA(swap_out);
 	STRESS_VMSTAT_DELTA(block_in);
@@ -577,6 +838,7 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
 	(void)memcpy(&vmstat_prev, &vmstat_current, sizeof(vmstat_prev));
 }
 
+#if defined(__linux__)
 /*
  *  stress_get_tz_info()
  *	get temperature in degrees C from a thermal zone
@@ -598,20 +860,38 @@ static double stress_get_tz_info(stress_tz_info_t *tz_info)
 	}
 	return temp;
 }
+#endif
 
+static void stress_zero_cpu_ghz(
+	double *avg_ghz,
+	double *min_ghz,
+	double *max_ghz)
+{
+	*avg_ghz = 0.0;
+	*min_ghz = 0.0;
+	*max_ghz = 0.0;
+}
+
+#if defined(__linux__)
 /*
- *  stress_get_cpu_ghz_average()
- *	compute average CPU frequencies in GHz
+ *  stress_get_cpu_ghz()
+ *	get CPU frequencies in GHz
  */
-static double stress_get_cpu_ghz_average(void)
+static void stress_get_cpu_ghz(
+	double *avg_ghz,
+	double *min_ghz,
+	double *max_ghz)
 {
 	struct dirent **cpu_list = NULL;
 	int i, n_cpus, n = 0;
 	double total_freq = 0.0;
 
+	*min_ghz = DBL_MAX;
+	*max_ghz = 0.0;
+
 	n_cpus = scandir("/sys/devices/system/cpu", &cpu_list, NULL, alphasort);
 	for (i = 0; i < n_cpus; i++) {
-		char *name = cpu_list[i]->d_name;
+		const char *name = cpu_list[i]->d_name;
 
 		if (!strncmp(name, "cpu", 3) && isdigit(name[3])) {
 			char path[PATH_MAX];
@@ -623,8 +903,14 @@ static double stress_get_cpu_ghz_average(void)
 				name);
 			if ((fp = fopen(path, "r")) != NULL) {
 				if (fscanf(fp, "%lf", &freq) == 1) {
-					total_freq += freq;
-					n++;
+					if (freq >= 0.0) {
+						total_freq += freq;
+						if (*min_ghz > freq)
+							*min_ghz = freq;
+						if (*max_ghz < freq)
+							*max_ghz = freq;
+						n++;
+					}
 				}
 				(void)fclose(fp);
 			}
@@ -634,8 +920,66 @@ static double stress_get_cpu_ghz_average(void)
 	if (n_cpus > -1)
 		free(cpu_list);
 
-	return (n == 0) ? 0.0 : (total_freq / n) / 1000000.0;
+	if (n == 0) {
+		stress_zero_cpu_ghz(avg_ghz, min_ghz, max_ghz);
+	} else {
+		*avg_ghz = (total_freq / n) * ONE_MILLIONTH;
+		*min_ghz *= ONE_MILLIONTH;
+		*max_ghz *= ONE_MILLIONTH;
+	}
 }
+#elif defined(__FreeBSD__) ||	\
+      defined(__APPLE__)
+static void stress_get_cpu_ghz(
+	double *avg_ghz,
+	double *min_ghz,
+	double *max_ghz)
+{
+	const int32_t ncpus = stress_get_processors_configured();
+	int32_t i;
+	double total_freq = 0.0;
+	int n = 0;
+
+	*min_ghz = DBL_MAX;
+	*max_ghz = 0.0;
+
+	for (i = 0; i < ncpus; i++) {
+		double freq;
+#if defined(__FreeBSD__)
+		{
+			char name[32];
+
+			(void)snprintf(name, sizeof(name), "dev.cpu.%" PRIi32 ".freq", i);
+			freq = (double)stress_bsd_getsysctl_uint(name) * ONE_THOUSANDTH;
+		}
+#endif
+#if defined(__APPLE__)
+		freq = (double)stress_bsd_getsysctl_uint64("hw.cpufrequency") * ONE_BILLIONTH;
+#endif
+		if (freq >= 0.0) {
+			total_freq += freq;
+			if (*min_ghz > freq)
+				*min_ghz = freq;
+			if (*max_ghz < freq)
+				*max_ghz = freq;
+			n++;
+		}
+	}
+	if (n == 0) {
+		stress_zero_cpu_ghz(avg_ghz, min_ghz, max_ghz);
+	} else {
+		*avg_ghz = (total_freq / n);
+	}
+}
+#else
+static void stress_get_cpu_ghz(
+	double *avg_ghz,
+	double *min_ghz,
+	double *max_ghz)
+{
+	stress_zero_cpu_ghz(avg_ghz, min_ghz, max_ghz);
+}
+#endif
 
 /*
  *  stress_vmstat_start()
@@ -644,18 +988,21 @@ static double stress_get_cpu_ghz_average(void)
 void stress_vmstat_start(void)
 {
 	stress_vmstat_t vmstat;
-	stress_iostat_t iostat;
 	size_t tz_num = 0;
-	stress_tz_info_t *tz_info, *tz_info_list;
+	stress_tz_info_t *tz_info;
 	int32_t vmstat_sleep, thermalstat_sleep, iostat_sleep;
+	double t1, t2;
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 	char iostat_name[PATH_MAX];
+	stress_iostat_t iostat;
+#endif
 
 	if ((vmstat_delay == 0) &&
 	    (thermalstat_delay == 0) &&
 	    (iostat_delay == 0))
 		return;
 
-	tz_info_list = NULL;
 	vmstat_sleep = vmstat_delay;
 	thermalstat_sleep = thermalstat_delay;
 	iostat_sleep = iostat_delay;
@@ -668,30 +1015,46 @@ void stress_vmstat_start(void)
 		stress_get_vmstat(&vmstat);
 
 	if (thermalstat_delay) {
-		stress_tz_init(&tz_info_list);
-
-		for (tz_info = tz_info_list; tz_info; tz_info = tz_info->next)
+		for (tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next)
 			tz_num++;
 	}
 
-
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 	if (stress_iostat_iostat_name(iostat_name, sizeof(iostat_name)) == NULL)
 		iostat_sleep = 0;
 	if (iostat_delay)
 		stress_get_iostat(iostat_name, &iostat);
+#endif
+
+#if defined(SCHED_DEADLINE)
+	VOID_RET(int, stress_set_sched(getpid(), SCHED_DEADLINE, 99, true));
+#endif
+
+	t1 = stress_time_now();
 
 	while (keep_stressing_flag()) {
 		int32_t sleep_delay = INT_MAX;
 		long clk_tick;
+		double delta;
 
 		if (vmstat_delay > 0)
 			sleep_delay = STRESS_MINIMUM(vmstat_delay, sleep_delay);
 		if (thermalstat_delay > 0)
 			sleep_delay = STRESS_MINIMUM(thermalstat_delay, sleep_delay);
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 		if (iostat_delay > 0)
 			sleep_delay = STRESS_MINIMUM(iostat_delay, sleep_delay);
+#endif
+		t1 += sleep_delay;
+		t2 = stress_time_now();
 
-		(void)sleep((unsigned int)sleep_delay);
+		delta = t1 - t2;
+		if (delta > 0) {
+			uint64_t nsec = (uint64_t)(delta * STRESS_DBL_NANOSECOND);
+			(void)shim_nanosleep_uint64(nsec);
+		}
 
 		/* This may change each time we get stats */
 		clk_tick = sysconf(_SC_CLK_TCK) * sysconf(_SC_NPROCESSORS_ONLN);
@@ -712,7 +1075,7 @@ void stress_vmstat_start(void)
 			static uint32_t vmstat_count = 0;
 
 			if ((vmstat_count++ % 25) == 0)
-				pr_inf("vmstat %2s %2s %9s %9s %9s %9s "
+				pr_inf("vmstat: %2s %2s %9s %9s %9s %9s "
 					"%4s %4s %6s %6s %4s %4s %2s %2s "
 					"%2s %2s %2s\n",
 					"r", "b", "swpd", "free", "buff",
@@ -721,7 +1084,7 @@ void stress_vmstat_start(void)
 					"wa", "st");
 
 			stress_get_vmstat(&vmstat);
-			pr_inf("vmstat %2" PRIu64 " %2" PRIu64 /* procs */
+			pr_inf("vmstat: %2" PRIu64 " %2" PRIu64 /* procs */
 			       " %9" PRIu64 " %9" PRIu64	/* vm used */
 			       " %9" PRIu64 " %9" PRIu64	/* memory_buff */
 			       " %4" PRIu64 " %4" PRIu64	/* si, so*/
@@ -732,10 +1095,10 @@ void stress_vmstat_start(void)
 			       " %2.0f\n",			/* st */
 				vmstat.procs_running,
 				vmstat.procs_blocked,
-				vmstat.swap_total - vmstat.swap_used,
+				vmstat.swap_used,
 				vmstat.memory_free,
 				vmstat.memory_buff,
-				vmstat.memory_cache,
+				vmstat.memory_cached + vmstat.memory_reclaimable,
 				vmstat.swap_in / (uint64_t)vmstat_delay,
 				vmstat.swap_out / (uint64_t)vmstat_delay,
 				vmstat.block_in / (uint64_t)vmstat_delay,
@@ -750,51 +1113,63 @@ void stress_vmstat_start(void)
 		}
 
 		if (thermalstat_delay == thermalstat_sleep) {
-			double min1, min5, min15, ghz;
-			char therms[1 + (tz_num * 6)];
-			char cpuspeed[6];
+			double min1, min5, min15, avg_ghz, min_ghz, max_ghz;
+			size_t therms_len = 1 + (tz_num * 7);
+			char *therms;
+			char cpuspeed[19];
+#if defined(__linux__)
 			char *ptr;
+#endif
 			static uint32_t thermalstat_count = 0;
 
-			(void)memset(therms, 0, sizeof(therms));
+			therms = calloc(therms_len, sizeof(*therms));
+			if (therms) {
+#if defined(__linux__)
+				for (ptr = therms, tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next) {
+					(void)snprintf(ptr, 8, " %6.6s", tz_info->type);
+					ptr += 7;
+				}
+#endif
+				if ((thermalstat_count++ % 25) == 0)
+					pr_inf("therm: AvGHz MnGhz MxGHz  LdA1  LdA5 LdA15 %s\n", therms);
 
-			for (ptr = therms, tz_info = tz_info_list; tz_info; tz_info = tz_info->next) {
-				(void)snprintf(ptr, 8, " %6.6s", tz_info->type);
-				ptr += 7;
-			}
+#if defined(__linux__)
+				for (ptr = therms, tz_info = g_shared->tz_info; tz_info; tz_info = tz_info->next) {
+					(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
+					ptr += 7;
+				}
+#endif
+				stress_get_cpu_ghz(&avg_ghz, &min_ghz, &max_ghz);
+				if (avg_ghz > 0.0)
+					(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.2f %5.2f %5.2f",
+						avg_ghz, min_ghz, max_ghz);
+				else
+					(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.5s %5.5s %5.5s",
+						" n/a ", " n/a ", " n/a ");
 
-			if ((thermalstat_count++ % 25) == 0)
-				pr_inf("therm:   GHz  LdA1  LdA5 LdA15 %s\n", therms);
-
-			for (ptr = therms, tz_info = tz_info_list; tz_info; tz_info = tz_info->next) {
-				(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
-				ptr += 7;
-			}
-			ghz = stress_get_cpu_ghz_average();
-			if (ghz > 0.0)
-				(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.2f", ghz);
-			else
-				(void)shim_strlcpy(cpuspeed, "n/a", sizeof(cpuspeed));
-
-			if (stress_get_load_avg(&min1, &min5, &min15) < 0)  {
-				pr_inf("therm: %5s %5.5s %5.5s %5.5s %s\n",
-					cpuspeed, "n/a", "n/a", "n/a", therms);
-			} else {
-				pr_inf("therm: %5s %5.2f %5.2f %5.2f %s\n",
-					cpuspeed, min1, min5, min15, therms);
+				if (stress_get_load_avg(&min1, &min5, &min15) < 0)  {
+					pr_inf("therm: %18s %5.5s %5.5s %5.5s %s\n",
+						cpuspeed, "n/a", "n/a", "n/a", therms);
+				} else {
+					pr_inf("therm: %5s %5.2f %5.2f %5.2f %s\n",
+						cpuspeed, min1, min5, min15, therms);
+				}
+				free(therms);
 			}
 		}
 
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 		if (iostat_delay == iostat_sleep) {
 			double clk_scale = (iostat_delay > 0) ? 1.0 / iostat_delay : 0.0;
 			static uint32_t iostat_count = 0;
 
 			if ((iostat_count++ % 25) == 0)
-				pr_inf("iostat: Inflght  Rd K/s   Wr K/s Dscd K/s     Rd/s     Wr/s   Dscd/s\n");
+				pr_inf("iostat: Inflght   Rd K/s   Wr K/s Dscd K/s     Rd/s     Wr/s   Dscd/s\n");
 
 			stress_get_iostat(iostat_name, &iostat);
 			/* sectors are 512 bytes, so >> 1 to get stats in 1024 bytes */
-			pr_inf("iostat %7.0f %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f\n",
+			pr_inf("iostat: %7.0f %8.0f %8.0f %8.0f %8.0f %8.0f %8.0f\n",
 				(double)iostat.in_flight * clk_scale,
 				(double)(iostat.read_sectors >> 1) * clk_scale,
 				(double)(iostat.write_sectors >> 1) * clk_scale,
@@ -803,7 +1178,9 @@ void stress_vmstat_start(void)
 				(double)iostat.write_io * clk_scale,
 				(double)iostat.discard_io * clk_scale);
 		}
+#endif
 	}
+	_exit(0);
 }
 
 /*
@@ -819,18 +1196,3 @@ void stress_vmstat_stop(void)
 		(void)waitpid(vmstat_pid, &status, 0);
 	}
 }
-
-#else
-
-/*
- *  no-ops for non-linux cases
- */
-void stress_vmstat_start(void)
-{
-}
-
-void stress_vmstat_stop(void)
-{
-}
-
-#endif
