@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,13 +31,13 @@ typedef struct {
 } stress_affinity_info_t;
 
 static const stress_help_t help[] = {
-	{ NULL,	"affinity N",	  "start N workers that rapidly change CPU affinity" },
-	{ NULL,	"affinity-ops N", "stop after N affinity bogo operations" },
-	{ NULL,	"affinity-rand",  "change affinity randomly rather than sequentially" },
-	{ NULL, "affinity-delay", "delay in nanoseconds between affinity changes" },
-	{ NULL, "affinity-pin",   "keep per stressor threads pinned to same CPU" },
-	{ NULL,	"affinity-sleep", "sleep in nanoseconds between affinity changes" },
-	{ NULL,	NULL,		  NULL }
+	{ NULL,	"affinity N",	  	"start N workers that rapidly change CPU affinity" },
+	{ NULL, "affinity-delay D",	"delay in nanoseconds between affinity changes" },
+	{ NULL,	"affinity-ops N",	"stop after N affinity bogo operations" },
+	{ NULL, "affinity-pin",		"keep per stressor threads pinned to same CPU" },
+	{ NULL,	"affinity-rand",	"change affinity randomly rather than sequentially" },
+	{ NULL,	"affinity-sleep",	"sleep in nanoseconds between affinity changes" },
+	{ NULL,	NULL,			NULL }
 };
 
 static int stress_set_affinity_delay(const char *opt)
@@ -52,18 +52,12 @@ static int stress_set_affinity_delay(const char *opt)
 
 static int stress_set_affinity_rand(const char *opt)
 {
-	bool affinity_rand = true;
-
-	(void)opt;
-	return stress_set_setting("affinity-rand", TYPE_ID_BOOL, &affinity_rand);
+	return stress_set_setting_true("affinity-rand", opt);
 }
 
 static int stress_set_affinity_pin(const char *opt)
 {
-	bool affinity_pin = true;
-
-	(void)opt;
-	return stress_set_setting("affinity-pin", TYPE_ID_BOOL, &affinity_pin);
+	return stress_set_setting_true("affinity-pin", opt);
 }
 
 static int stress_set_affinity_sleep(const char *opt)
@@ -88,8 +82,8 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  stress on sched_affinity()
  *	stress system by changing CPU affinity periodically
  */
-#if defined(HAVE_AFFINITY) && \
-    defined(HAVE_SCHED_GETAFFINITY)
+#if defined(HAVE_SCHED_GETAFFINITY) &&	\
+    defined(HAVE_SCHED_SETAFFINITY)
 
 static void *counter_lock;	/* Counter lock */
 
@@ -254,7 +248,7 @@ static int stress_affinity(const stress_args_t *args)
 
 	counter_lock = stress_lock_create();
 	if (!counter_lock) {
-		pr_inf("%s: failed to create counter lock. skipping stressor\n", args->name);
+		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -320,9 +314,10 @@ stressor_info_t stress_affinity_info = {
 };
 #else
 stressor_info_t stress_affinity_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sched_getaffinity() or sched_setaffinity()"
 };
 #endif
