@@ -17,6 +17,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-cache.h"
 
 #define DEFAULT_L1_SIZE		(64)
 
@@ -26,45 +28,21 @@
 	do { __atomic_fetch_add(ptr, 1, __ATOMIC_RELAXED); } while (0)
 #endif
 
-/*
- *  8 bit rotate right
- */
-#define ROR8(val)				\
-do {						\
-	uint8_t tmpval = (val);			\
-	const uint8_t bit0 = (tmpval & 1) << 7;	\
-	tmpval >>= 1;				\
-	tmpval |= bit0;				\
-	(val) = tmpval;				\
-} while (0)
-
-/*
- *  8 bit rotate left
- */
-#define ROL8(val)				\
-do {						\
-	uint8_t tmpval = (val);			\
-	const uint8_t bit7 = (tmpval & 0x80) >> 7;\
-	tmpval <<= 1;				\
-	tmpval |= bit7;				\
-	(val) = tmpval;				\
-} while (0)
-
-#define EXERCISE(data)	\
-do {			\
-	(data)++;	\
-	shim_mb();	\
-	ROL8(data);	\
-	shim_mb();	\
-	ROR8(data);	\
-	shim_mb();	\
+#define EXERCISE(data)		\
+do {				\
+	(data)++;		\
+	shim_mb();		\
+	data = shim_rol8(data);	\
+	shim_mb();		\
+	data = shim_ror8(data);	\
+	shim_mb();		\
 } while (0)
 
 static const stress_help_t help[] = {
 	{ NULL,	"cacheline N",		"start N workers that exercise cachelines" },
-	{ NULL,	"cacheline-ops N",	"stop after N cacheline bogo operations" },
 	{ NULL,	"cacheline-affinity",	"modify CPU affinity" },
 	{ NULL,	"cacheline-method M",	"use cacheline stressing method M" },
+	{ NULL,	"cacheline-ops N",	"stop after N cacheline bogo operations" },
 	{ NULL,	NULL,			NULL }
 };
 
@@ -82,7 +60,8 @@ typedef struct {
 static uint64_t get_L1_line_size(const stress_args_t *args)
 {
 	uint64_t cache_size = DEFAULT_L1_SIZE;
-#if defined(__linux__)
+#if defined(__linux__) ||	\
+    defined(__APPLE__)
 	stress_cpus_t *cpu_caches;
 	stress_cpu_cache_t *cache = NULL;
 
@@ -177,13 +156,14 @@ static int stress_cacheline_copy(
 	register int i;
 	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
 	volatile uint8_t *data8 = cacheline + index;
-	register uint8_t val8;
 	const volatile uint8_t *data8adjacent = (volatile uint8_t *)(((uintptr_t)data8) ^ 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
 	for (i = 0; i < 1024; i++) {
+		register uint8_t val8;
+
 		(*data8) = (*data8adjacent);
 		(*data8) = (*data8adjacent);
 		(*data8) = (*data8adjacent);
@@ -258,53 +238,55 @@ static int stress_cacheline_rdwr(
 	(void)l1_cacheline_size;
 
 	for (i = 0; i < 1024; i++) {
-		(void)*data8;
-		*data8 = *data8;
+		uint8_t tmp;
+
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
-		(void)*data8;
-		*data8 = *data8;
+		tmp = *data8;
+		*data8 = tmp;
 		shim_mb();
 
 		if (UNLIKELY(*data8 != val8)) {
@@ -323,7 +305,7 @@ static int stress_cacheline_mix(
 	const size_t l1_cacheline_size)
 {
 	register int i;
-	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
+	volatile uint8_t *cacheline = (volatile uint8_t *)(uintptr_t)g_shared->cacheline;
 	volatile uint8_t *data8 = cacheline + index;
 	static uint8_t tmp = 0xa5;
 
@@ -345,7 +327,7 @@ static int stress_cacheline_mix(
 		tmp = val8;
 	}
 	return EXIT_SUCCESS;
-};
+}
 
 static int stress_cacheline_rdrev64(
 	const stress_args_t *args,
@@ -356,9 +338,8 @@ static int stress_cacheline_rdrev64(
 	register int i;
 	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
 	volatile uint8_t *data8 = cacheline + index;
-	const size_t cacheline_size = g_shared->cacheline_size;
-	volatile uint8_t *aligned_cacheline = (volatile uint8_t *)
-		((intptr_t)cacheline & ~(l1_cacheline_size - 1));
+	const ssize_t cacheline_size = (ssize_t)g_shared->cacheline_size;
+	uintptr_t aligned_cacheline = (uintptr_t)cacheline & ~(l1_cacheline_size - 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
@@ -371,8 +352,8 @@ static int stress_cacheline_rdrev64(
 		val8 = *data8;
 
 		/* read cache line backwards */
-		for (j = (ssize_t)cacheline_size - 8; j >= 0; j -= 8) {
-			volatile uint64_t *data64 = (volatile uint64_t *)(aligned_cacheline + j);
+		for (j = cacheline_size - 8; j >= 0; j -= 8) {
+			volatile uint64_t *data64 = (volatile uint64_t *)(aligned_cacheline + (size_t)j);
 
 			(void)*data64;
 			shim_mb();
@@ -396,8 +377,7 @@ static int stress_cacheline_rdfwd64(
 	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
 	volatile uint8_t *data8 = cacheline + index;
 	const size_t cacheline_size = g_shared->cacheline_size;
-	volatile uint8_t *aligned_cacheline = (volatile uint8_t *)
-		((intptr_t)cacheline & ~(l1_cacheline_size - 1));
+	uintptr_t aligned_cacheline = (uintptr_t)cacheline & ~(l1_cacheline_size - 1);
 
 	(void)parent;
 	(void)l1_cacheline_size;
@@ -487,15 +467,17 @@ static int stress_cacheline_bits(
 	register int i;
 	volatile uint8_t *cacheline = (volatile uint8_t *)g_shared->cacheline;
 	volatile uint8_t *data8 = cacheline + index;
-	register uint8_t val8;
 
 	(void)parent;
 	(void)l1_cacheline_size;
 
 	for (i = 0; i < 1024; i++) {
-		(void)*(data8);
+		register uint8_t val8;
 
-		val8 = 1U << (i & 7);
+		val8 = *(data8);
+		(void)val8;
+
+		val8 = (uint8_t)(1U << (i & 7));
 		*data8 = val8;
 		shim_mb();
 		if (*data8 != val8) {
@@ -593,10 +575,7 @@ static int stress_cacheline_all(
 
 static int stress_set_cacheline_affinity(const char *opt)
 {
-	bool cacheline_affinity = true;
-
-	(void)opt;
-	return stress_set_setting("cacheline-affinity", TYPE_ID_BOOL, &cacheline_affinity);
+	return stress_set_setting_true("cacheline-affinity", opt);
 }
 
 /*
@@ -623,8 +602,8 @@ static int stress_set_cacheline_method(const char *name)
 	return -1;
 }
 
-#if defined(HAVE_AFFINITY) && \
-    defined(HAVE_SCHED_GETAFFINITY)
+#if defined(HAVE_SCHED_GETAFFINITY) &&	\
+    defined(HAVE_SCHED_SETAFFINITY)
 /*
  *  stress_cacheline_change_affinity()
  *	pin process to CPU based on clock time * 100, instance number
@@ -654,9 +633,9 @@ static int stress_cacheline_child(
 	const bool cacheline_affinity)
 {
 	int rc;
-#if defined(HAVE_AFFINITY) && \
-    defined(HAVE_SCHED_GETAFFINITY)
-	const uint32_t cpus = (int)stress_get_processors_configured();
+#if defined(HAVE_SCHED_GETAFFINITY) &&	\
+    defined(HAVE_SCHED_SETAFFINITY)
+	const uint32_t cpus = (uint32_t)stress_get_processors_configured();
 #endif
 
 	(void)cacheline_affinity;
@@ -666,8 +645,8 @@ static int stress_cacheline_child(
 		if (parent)
 			inc_counter(args);
 
-#if defined(HAVE_AFFINITY) && \
-    defined(HAVE_SCHED_GETAFFINITY)
+#if defined(HAVE_SCHED_GETAFFINITY) &&	\
+    defined(HAVE_SCHED_SETAFFINITY)
 		if (cacheline_affinity)
 			stress_cacheline_change_affinity(args, cpus, parent);
 #endif
@@ -698,6 +677,7 @@ static int stress_cacheline(const stress_args_t *args)
 	(void)stress_get_setting("cacheline-method", &cacheline_method);
 
 	if (args->instance == 0) {
+		pr_dbg("%s: using method '%s'\n", args->name, cacheline_methods[cacheline_method].name);
 		pr_dbg("%s: L1 cache line size %zd bytes\n", args->name, l1_cacheline_size);
 
 		if ((args->num_instances * 2) < l1_cacheline_size) {
@@ -706,7 +686,6 @@ static int stress_cacheline(const stress_args_t *args)
 		}
 	}
 
-	pr_dbg("%s: using method '%s'\n", args->name, cacheline_methods[cacheline_method].name);
 	func = cacheline_methods[cacheline_method].func;
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
