@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -145,7 +145,7 @@ static int stress_chown(const stress_args_t *args)
 	stress_temp_dir(pathname, sizeof(pathname), args->name, ppid, 0);
 	if (mkdir(pathname, S_IRUSR | S_IRWXU) < 0) {
 		if (errno != EEXIST) {
-			rc = exit_status(errno);
+			rc = stress_exit_status(errno);
 			pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
 				args->name, pathname, errno, strerror(errno));
 			return rc;
@@ -156,7 +156,7 @@ static int stress_chown(const stress_args_t *args)
 
 	if (args->instance == 0) {
 		if ((fd = creat(filename, S_IRUSR | S_IWUSR)) < 0) {
-			rc = exit_status(errno);
+			rc = stress_exit_status(errno);
 			pr_fail("%s: creat %s failed, errno=%d (%s)\n",
 				args->name, filename, errno, strerror(errno));
 			goto tidy;
