@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -259,13 +259,13 @@ static int stress_chroot_test7(
 	const size_t path_len = 256 * KB;
 	int ret1, ret2, errno1, errno2;
 	char *path;
-	
+
 	/* Don't throw a failure of we can't allocate large path */
 	path = malloc(path_len);
 	if (!path)
 		return EXIT_SUCCESS;
 
-	stress_strnrnd(path, path_len);
+	stress_rndstr(path, path_len);
 	path[0] = '/';
 
 	do_chroot(path, &ret1, &ret2, &errno1, &errno2, metrics);
@@ -313,7 +313,7 @@ static int stress_chroot(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	stress_strnrnd(longpath, sizeof(longpath));
+	stress_rndstr(longpath, sizeof(longpath));
 	(void)stress_temp_dir(badpath, sizeof(badpath), "badpath", args->pid, 0xbad);
 	(void)stress_temp_dir_args(args, temppath, sizeof(temppath));
 	(void)stress_temp_filename_args(args, filename, sizeof(filename), stress_mwc32());
@@ -368,7 +368,7 @@ again:
 	} while (keep_stressing(args));
 
 	rate = (metrics->duration > 0.0) ? metrics->count / metrics->duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "chroot calls per sec", rate);
+	stress_metrics_set(args, 0, "chroot calls per sec", rate);
 
 	ret = EXIT_SUCCESS;
 tidy_all:
