@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -218,9 +218,9 @@ static int stress_copy_file(const stress_args_t *args)
 		shim_loff_t off_in, off_out, off_in_orig, off_out_orig;
 		double t;
 
-		off_in_orig = (shim_loff_t)(stress_mwc64() % (copy_file_bytes - DEFAULT_COPY_FILE_SIZE));
+		off_in_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
 		off_in = off_in_orig;
-		off_out_orig = (shim_loff_t)(stress_mwc64() % (copy_file_bytes - DEFAULT_COPY_FILE_SIZE));
+		off_out_orig = (shim_loff_t)stress_mwc64modn(copy_file_bytes - DEFAULT_COPY_FILE_SIZE);
 		off_out = off_out_orig;
 
 		if (!keep_stressing(args))
@@ -289,7 +289,7 @@ static int stress_copy_file(const stress_args_t *args)
 	rc = EXIT_SUCCESS;
 
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "MB per sec copy rate", rate / (double)MB);
+	stress_metrics_set(args, 0, "MB per sec copy rate", rate / (double)MB);
 
 tidy_out:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
