@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -50,7 +45,7 @@ static int stress_cpu_online_set(
 	(void)snprintf(filename, sizeof(filename),
 		"/sys/devices/system/cpu/cpu%" PRIu32 "/online", cpu);
 
-	ret = system_write(filename, data, sizeof data);
+	ret = system_write(filename, data, sizeof(data));
 	if ((ret < 0) &&
 	    ((ret != -EAGAIN) && (ret != -EINTR) &&
 	     (ret != -EBUSY) && (ret != -EOPNOTSUPP))) {
@@ -61,6 +56,40 @@ static int stress_cpu_online_set(
 	return EXIT_SUCCESS;
 }
 
+/*
+ *  stress_cpu_online_get()
+ *	get a specified CPUs online or offline state
+ */
+static int stress_cpu_online_get(const uint32_t cpu, int *setting)
+{
+	char filename[PATH_MAX];
+	char data[3];
+	ssize_t ret;
+
+	(void)snprintf(filename, sizeof(filename),
+		"/sys/devices/system/cpu/cpu%" PRIu32 "/online", cpu);
+
+	(void)memset(data, 0, sizeof(data));
+	ret = system_read(filename, data, sizeof(data));
+	if (ret < 1) {
+		*setting = -1;
+		return EXIT_FAILURE;
+	}
+	switch (data[0]) {
+	case '0':
+		*setting = 0;
+		break;
+	case '1':
+		*setting = 1;
+		break;
+	default:
+		*setting = -1;
+		return EXIT_FAILURE;
+	}
+	return EXIT_SUCCESS;
+}
+
+
 /*
  *  stress_cpu_online_supported()
  *      check if we can run this as root
@@ -95,6 +124,9 @@ static int stress_cpu_online(const stress_args_t *args)
 	int32_t i, cpu_online_count = 0;
 	bool *cpu_online;
 	int rc = EXIT_SUCCESS;
+	double offline_duration = 0.0, offline_count = 0.0;
+	double online_duration  = 0.0, online_count = 0.0;
+	double rate;
 
 	if (geteuid() != 0) {
 		if (args->instance == 0)
@@ -118,7 +150,8 @@ static int stress_cpu_online(const stress_args_t *args)
 
 	cpu_online = calloc((size_t)cpus, sizeof(*cpu_online));
 	if (!cpu_online) {
-		pr_err("%s: out of memory\n", args->name);
+		pr_inf_skip("%s: out of memory allocating %" PRId32 " boolean flags, "
+			    "skipping stressor\n", args->name, cpus);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -165,7 +198,8 @@ static int stress_cpu_online(const stress_args_t *args)
 	 *  Now randomly offline/online them all
 	 */
 	do {
-		uint32_t cpu = stress_mwc32() % (uint32_t)cpus;
+		const uint32_t cpu = stress_mwc32modn((uint32_t)cpus);
+		double t;
 
 		/*
 		 * Only allow CPU 0 to be offlined if OPT_FLAGS_CPU_ONLINE_ALL
@@ -174,12 +208,32 @@ static int stress_cpu_online(const stress_args_t *args)
 		if ((cpu == 0) && !(g_opt_flags & OPT_FLAGS_CPU_ONLINE_ALL))
 			continue;
 		if (cpu_online[cpu]) {
+			int setting;
+
+			t = stress_time_now();
 			rc = stress_cpu_online_set(args, cpu, 0);
 			if (rc != EXIT_SUCCESS)
 				break;
+			rc = stress_cpu_online_get(cpu, &setting);
+			if ((rc == EXIT_SUCCESS) && (setting != 0)) {
+				pr_inf("%s: set cpu offline, expecting setting to be 0, got %d instead\n",
+					args->name, setting);
+			} else {
+				offline_duration += stress_time_now() - t;
+				offline_count += 1.0;
+			}
+			t = stress_time_now();
 			rc = stress_cpu_online_set(args, cpu, 1);
 			if (rc != EXIT_SUCCESS)
 				break;
+			rc = stress_cpu_online_get(cpu, &setting);
+			if ((rc == EXIT_SUCCESS) && (setting != 1)) {
+				pr_inf("%s: set cpu offline, expecting setting to be 1, got %d instead\n",
+					args->name, setting);
+			} else {
+				online_duration += stress_time_now() - t;
+				online_count += 1.0;
+			}
 			inc_counter(args);
 		}
 	} while (keep_stressing(args));
@@ -195,6 +249,11 @@ static int stress_cpu_online(const stress_args_t *args)
 	}
 	free(cpu_online);
 
+	rate = (offline_count > 0.0) ? (double)offline_duration / offline_count : 0.0;
+	stress_metrics_set(args, 0, "millisecs per offline action", rate * STRESS_DBL_MILLISECOND);
+	rate = (online_count > 0.0) ? (double)online_duration / online_count : 0.0;
+	stress_metrics_set(args, 1, "millisecs per online action", rate * STRESS_DBL_MILLISECOND);
+
 	return rc;
 }
 
@@ -202,12 +261,14 @@ stressor_info_t stress_cpu_online_info = {
 	.stressor = stress_cpu_online,
 	.supported = stress_cpu_online_supported,
 	.class = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_cpu_online_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "only supported on Linux"
 };
 #endif
