@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -251,7 +251,11 @@ static void OPTIMIZE3 TARGET_CLONES stress_cpu_bitops(const char *name)
 
 	for (i = 0; i < 16384; i++) {
 		{
-			register uint32_t r, v, s = (sizeof(v) * 8) - 1;
+			register uint32_t r;
+#if defined(HAVE_BUILTIN_BITREVERSE)
+			r = __builtin_bitreverse32(i);
+#else
+			register uint32_t v, s = (sizeof(v) * 8) - 1;
 
 			/* Reverse bits */
 			r = v = i;
@@ -260,6 +264,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_cpu_bitops(const char *name)
 				r |= v & 1;
 			}
 			r <<= s;
+#endif
 			i_sum += r;
 		}
 		{
@@ -440,6 +445,16 @@ static void HOT OPTIMIZE3 stress_cpu_logmap(const char *name)
 #if defined(HAVE_SRAND48) &&	\
     defined(HAVE_LRAND48) &&	\
     defined(HAVE_DRAND48)
+
+/*
+ * Some libc's such as OpenBSD don't implement rand48 the same
+ * as linux's glibc, so beware of different implementations with
+ * the verify mode checking.
+ */
+#if defined(__linux__) && 	\
+    defined(__GLIBC__)
+#define STRESS_CPU_RAND48_VERIFY
+#endif
 /*
  *  stress_cpu_rand48()
  *	generate random values using rand48 family of functions
@@ -448,7 +463,11 @@ static void HOT OPTIMIZE3 stress_cpu_rand48(const char *name)
 {
 	int i;
 	double d = 0;
-	long int l = 0;
+	long long int l = 0;
+#if defined(STRESS_CPU_RAND48_VERIFY)
+	double d_expected_sum = 8184.618041;
+	long long int l_expected_sum = 17522760427916;
+#endif
 
 	(void)name;
 
@@ -457,6 +476,17 @@ static void HOT OPTIMIZE3 stress_cpu_rand48(const char *name)
 		d += drand48();
 		l += lrand48();
 	}
+
+#if defined(STRESS_CPU_RAND48_VERIFY)
+	if (g_opt_flags & OPT_FLAGS_VERIFY) {
+		double d_error = d - d_expected_sum;
+		if (fabs(d_error) > 0.0001)
+			pr_fail("%s: drand48 error detected, failed sum\n", name);
+		if (l != l_expected_sum)
+			pr_fail("%s: lrand48 error detected, failed sum\n", name);
+	}
+#endif
+
 	stress_double_put(d);
 	stress_uint64_put((uint64_t)l);
 }
@@ -534,8 +564,8 @@ static void OPTIMIZE3 TARGET_CLONES stress_cpu_phi(const char *name)
 	int i;
 
 	/* Pick any two starting points */
-	a = stress_mwc64() % 99;
-	b = stress_mwc64() % 99;
+	a = stress_mwc64modn(99);
+	b = stress_mwc64modn(99);
 
 	/* Iterate until we approach overflow */
 	for (i = 0; (i < 64) && !((a | b) & mask); i++) {
@@ -2934,7 +2964,8 @@ static int HOT OPTIMIZE3 stress_cpu(const stress_args_t *args)
 			stress_cpu_counter_scale[i] = 1484.50 / cpu_methods[i].bogo_op_rate;
 	}
 
-	pr_dbg("%s: using method '%s'\n", args->name, cpu_methods[cpu_method].name);
+	if (args->instance == 0)
+		pr_dbg("%s: using method '%s'\n", args->name, cpu_methods[cpu_method].name);
 
 	/*
 	 * It is unlikely, but somebody may request to do a zero
@@ -2996,7 +3027,7 @@ static int HOT OPTIMIZE3 stress_cpu(const stress_args_t *args)
 			} while (t2 < slice_end);
 		} else {
 			/* > 0, time slice in milliseconds */
-			const double slice_end = t1 + ((double)cpu_load_slice / 1000.0);
+			const double slice_end = t1 + ((double)cpu_load_slice / STRESS_DBL_MILLISECOND);
 
 			do {
 				stress_cpu_method(cpu_method, args, &counter);
@@ -3025,10 +3056,10 @@ static int HOT OPTIMIZE3 stress_cpu(const stress_args_t *args)
 
 #if defined(HAVE_SELECT)
 			tv.tv_sec = (time_t)delay;
-			tv.tv_usec = (long)((delay - (double)tv.tv_sec) * 1000000.0);
+			tv.tv_usec = (long)((delay - (double)tv.tv_sec) * STRESS_DBL_MICROSECOND);
 			(void)select(0, NULL, NULL, NULL, &tv);
 #else
-			shim_nanosleep_uint64((uint64_t)(delay * 1000000000.0));
+			shim_nanosleep_uint64((uint64_t)(delay * STRESS_DBL_NANOSECOND));
 #endif
 
 			t3 = stress_time_now();
