@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -402,7 +402,7 @@ tidy_mmap:
 	}
 #endif
 	rate = (dup_count > 0.0) ? (double)dup_duration / dup_count : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "nanosecs per dup call", rate * 1000000000.0);
+	stress_metrics_set(args, 0, "nanosecs per dup call", rate * STRESS_DBL_NANOSECOND);
 
 	return rc;
 }
