@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2012-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -616,11 +616,11 @@ static int stress_fanotify(const stress_args_t *args)
 			free(buffer);
 			switch (-ret) {
 			case EMFILE:
-				pr_inf("%s: too many open files, skipping stressor\n", args->name);
+				pr_inf_skip("%s: too many open files, skipping stressor\n", args->name);
 				rc = EXIT_NO_RESOURCE;
 				break;
 			case ENOMEM:
-				pr_inf("%s: out of memory, skipping stressor\n", args->name);
+				pr_inf_skip("%s: out of memory, skipping stressor\n", args->name);
 				rc = EXIT_NO_RESOURCE;
 				break;
 			default:
@@ -717,6 +717,8 @@ tidy:
 		(void)shim_waitpid(pid, &status, 0);
 	}
 	(void)shim_unlink(filename);
+	(void)shim_unlink(filename2);
+	(void)sync();
 	(void)stress_temp_dir_rm_args(args);
 	stress_mount_free(mnts, n_mnts);
 
@@ -732,8 +734,9 @@ stressor_info_t stress_fanotify_info = {
 };
 #else
 stressor_info_t stress_fanotify_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/fanotify.h"
 };
 #endif
