@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -54,6 +54,7 @@ static int stress_full(const stress_args_t *args)
 		off_t offset;
 		char ALIGN64 buffer[4096];
 		uint8_t *ptr;
+		struct stat statbuf;
 
 		if ((fd = open("/dev/full", O_RDWR)) < 0) {
 			if (errno == ENOENT) {
@@ -88,10 +89,10 @@ static int stress_full(const stress_args_t *args)
 			return EXIT_FAILURE;
 		}
 
+try_read:
 		/*
 		 *  Reads should always work
 		 */
-try_read:
 		ret = read(fd, buffer, sizeof(buffer));
 		if (ret < 0) {
 			pr_fail("%s: read failed, errno=%d (%s)\n",
@@ -107,7 +108,27 @@ try_read:
 				return EXIT_FAILURE;
 			}
 		}
-
+#if defined(HAVE_PREAD)
+		{
+			offset = (sizeof(offset) == sizeof(uint64_t)) ?
+				(off_t)(stress_mwc64() & 0x7fffffffffffffff) :
+				(off_t)(stress_mwc32() & 0x7fffffffUL);
+			ret = pread(fd, buffer, sizeof(buffer), offset);
+			if (ret < 0) {
+				pr_fail("%s: read failed at offset %jd, errno=%d (%s)\n",
+					args->name, (intmax_t)offset, errno, strerror(errno));
+				(void)close(fd);
+				return EXIT_FAILURE;
+			}
+		}
+#endif
+		/*
+		 *  Try fstat
+		 */
+		ret = fstat(fd, &statbuf);
+		if (ret < 0)
+			pr_fail("%s: fstat failed, errno=%d (%s)\n",
+				args->name, errno, strerror(errno));
 		/*
 		 *  Try mmap'ing and msync on fd
 		 */
@@ -132,7 +153,7 @@ try_read:
 		/*
 		 *  Seeks will always succeed
 		 */
-		w = stress_mwc32() % 3;
+		w = stress_mwc32modn(3);
 		offset = (off_t)stress_mwc64();
 		ret = lseek(fd, offset, whences[w].whence);
 		if (ret < 0) {
@@ -158,8 +179,9 @@ stressor_info_t stress_full_info = {
 };
 #else
 stressor_info_t stress_full_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "only supported on Linux"
 };
 #endif
