@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -408,7 +408,7 @@ L0x000:
 
 	duration = t2 - t1;
 	rate = (duration > 0.0) ? (1024.0 * (double)get_counter(args)) / duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "million gotos per sec", rate / 1000000.0);
+	stress_metrics_set(args, 0, "million gotos per sec", rate / 1000000.0);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
