@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2017 Canonical, Ltd.
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+
+#if defined(HAVE_SYS_UIO_H)
 #include <sys/uio.h>
+#endif
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#endif
+
+#define MIN_HDD_BYTES		(1 * MB)
+#define MAX_HDD_BYTES		(MAX_FILE_LIMIT)
+#define DEFAULT_HDD_BYTES	(1 * GB)
+
+#define MIN_HDD_WRITE_SIZE	(1)
+#define MAX_HDD_WRITE_SIZE	(4 * MB)
+#define DEFAULT_HDD_WRITE_SIZE	(64 * 1024)
 
 #define BUF_ALIGNMENT		(4096)
 #define HDD_IO_VEC_MAX		(16)		/* Must be power of 2 */
@@ -50,7 +60,6 @@
 #define HDD_OPT_O_DSYNC		(0x00020000)
 #define HDD_OPT_O_DIRECT	(0x00040000)
 #define HDD_OPT_O_NOATIME	(0x00080000)
-#define HDD_OPT_O_MASK		(0x000f0000)
 
 /* Other modes */
 #define HDD_OPT_IOVEC		(0x00100000)
@@ -61,13 +70,22 @@
 
 typedef struct {
 	const char *opt;	/* User option */
-	int flag;		/* HDD_OPT_ flag */
-	int exclude;		/* Excluded HDD_OPT_ flags */
-	int advice;		/* posix_fadvise value */
-	int oflag;		/* open O_* flags */
-} hdd_opts_t;
+	const int flag;		/* HDD_OPT_ flag */
+	const int exclude;	/* Excluded HDD_OPT_ flags */
+	const int advice;	/* posix_fadvise value */	/* cppcheck-suppress unusedStructMember */
+	const int oflag;	/* open O_* flags */
+} stress_hdd_opts_t;
 
-static const hdd_opts_t hdd_opts[] = {
+static const stress_help_t help[] = {
+	{ "d N","hdd N",		"start N workers spinning on write()/unlink()" },
+	{ NULL,	"hdd-bytes N",		"write N bytes per hdd worker (default is 1GB)" },
+	{ NULL,	"hdd-ops N",		"stop after N hdd bogo operations" },
+	{ NULL,	"hdd-opts list",	"specify list of various stressor options" },
+	{ NULL,	"hdd-write-size N",	"set the default write size to N bytes" },
+	{ NULL, NULL,			NULL }
+};
+
+static const stress_hdd_opts_t hdd_opts[] = {
 #if defined(O_SYNC)
 	{ "sync",	HDD_OPT_O_SYNC, 0, 0, O_SYNC },
 #endif
@@ -80,7 +98,8 @@ static const hdd_opts_t hdd_opts[] = {
 #if defined(O_NOATIME)
 	{ "noatime",	HDD_OPT_O_NOATIME, 0, 0, O_NOATIME },
 #endif
-#if defined(POSIX_FADV_NORMAL) && !defined(__gnu_hurd__)
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    defined(POSIX_FADV_NORMAL)
 	{ "wr-seq",	HDD_OPT_WR_SEQ, HDD_OPT_WR_RND, 0, 0 },
 	{ "wr-rnd",	HDD_OPT_WR_RND, HDD_OPT_WR_SEQ, 0, 0 },
 	{ "rd-seq",	HDD_OPT_RD_SEQ, HDD_OPT_RD_RND, 0, 0 },
@@ -91,64 +110,95 @@ static const hdd_opts_t hdd_opts[] = {
 		 HDD_OPT_FADV_DONTNEED),
 		POSIX_FADV_NORMAL, 0 },
 #endif
-#if defined(POSIX_FADV_SEQ) && !defined(__gnu_hurd__)
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    defined(POSIX_FADV_SEQUENTIAL)
 	{ "fadv-seq",	HDD_OPT_FADV_SEQ,
 		(HDD_OPT_FADV_NORMAL | HDD_OPT_FADV_RND),
 		POSIX_FADV_SEQUENTIAL, 0 },
 #endif
-#if defined(POSIX_FADV_RANDOM) && !defined(__gnu_hurd__)
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    defined(POSIX_FADV_RANDOM)
 	{ "fadv-rnd",	HDD_OPT_FADV_RND,
 		(HDD_OPT_FADV_NORMAL | HDD_OPT_FADV_SEQ),
 		POSIX_FADV_RANDOM, 0 },
 #endif
-#if defined(POSIX_FADV_NOREUSE) && !defined(__gnu_hurd__)
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    defined(POSIX_FADV_NOREUSE)
 	{ "fadv-noreuse", HDD_OPT_FADV_NOREUSE,
 		HDD_OPT_FADV_NORMAL,
 		POSIX_FADV_NOREUSE, 0 },
 #endif
-#if defined(POSIX_FADV_WILLNEED) && !defined(__gnu_hurd__)
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    defined(POSIX_FADV_WILLNEED)
 	{ "fadv-willneed", HDD_OPT_FADV_WILLNEED,
 		(HDD_OPT_FADV_NORMAL | HDD_OPT_FADV_DONTNEED),
 		POSIX_FADV_WILLNEED, 0 },
 #endif
-#if defined(POSIX_FADV_DONTNEED) && !defined(__gnu_hurd__)
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    defined(POSIX_FADV_DONTNEED)
 	{ "fadv-dontneed", HDD_OPT_FADV_DONTNEED,
 		(HDD_OPT_FADV_NORMAL | HDD_OPT_FADV_WILLNEED),
 		POSIX_FADV_DONTNEED, 0 },
 #endif
-#if _BSD_SOURCE || _XOPEN_SOURCE || _POSIX_C_SOURCE >= 200112L
+#if defined(HAVE_FSYNC)
 	{ "fsync",	HDD_OPT_FSYNC, 0, 0, 0 },
 #endif
-#if _POSIX_C_SOURCE >= 199309L || _XOPEN_SOURCE >= 500
+#if defined(HAVE_FDATASYNC)
 	{ "fdatasync",	HDD_OPT_FDATASYNC, 0, 0, 0 },
 #endif
+#if defined(HAVE_SYS_UIO_H)
 	{ "iovec",	HDD_OPT_IOVEC, 0, 0, 0 },
-#if NEED_GLIBC(2,14,0) && defined(__linux__)
+#endif
+#if defined(HAVE_SYNCFS)
 	{ "syncfs",	HDD_OPT_SYNCFS, 0, 0, 0 },
 #endif
 	{ "utimes",	HDD_OPT_UTIMES, 0, 0, 0 },
 };
 
-void stress_set_hdd_bytes(const char *opt)
+static int stress_set_hdd_bytes(const char *opt)
 {
 	uint64_t hdd_bytes;
 
-	hdd_bytes = get_uint64_byte_filesystem(opt, 1);
-	check_range_bytes("hdd-bytes", hdd_bytes,
+	hdd_bytes = stress_get_uint64_byte_filesystem(opt, 1);
+	stress_check_range_bytes("hdd-bytes", hdd_bytes,
 		MIN_HDD_BYTES, MAX_HDD_BYTES);
-	set_setting("hdd-bytes", TYPE_ID_UINT64, &hdd_bytes);
+	return stress_set_setting("hdd-bytes", TYPE_ID_UINT64, &hdd_bytes);
 }
 
-void stress_set_hdd_write_size(const char *opt)
+static int stress_set_hdd_write_size(const char *opt)
 {
 	uint64_t hdd_write_size;
 
-	hdd_write_size = get_uint64_byte(opt);
-	check_range_bytes("hdd-write-size", hdd_write_size,
+	hdd_write_size = stress_get_uint64_byte(opt);
+	stress_check_range_bytes("hdd-write-size", hdd_write_size,
 		MIN_HDD_WRITE_SIZE, MAX_HDD_WRITE_SIZE);
-	set_setting("hdd-write-size", TYPE_ID_UINT64, &hdd_write_size);
+	return stress_set_setting("hdd-write-size", TYPE_ID_UINT64, &hdd_write_size);
 }
 
+#if defined(HAVE_FUTIMES)
+static void stress_hdd_utimes(const int fd)
+{
+	struct timeval tv[2];
+
+	(void)futimes(fd, NULL);
+
+	/* Exercise illegal futimes, usec too large */
+	tv[0].tv_usec = 1000001;
+	tv[0].tv_sec = 0;
+	tv[1].tv_usec = 1000001;
+	tv[1].tv_sec = 0;
+	(void)futimes(fd, tv);
+
+	/* Exercise illegal futimes, usec too small */
+	tv[0].tv_usec = -1;
+	tv[0].tv_sec = -1;
+	tv[1].tv_usec = -1;
+	tv[1].tv_sec = -1;
+
+	(void)futimes(fd, tv);
+}
+#endif
+
 /*
  *  stress_hdd_write()
  *	write with writev or write depending on mode
@@ -156,18 +206,21 @@ void stress_set_hdd_write_size(const char *opt)
 static ssize_t stress_hdd_write(
 	const int fd,
 	uint8_t *buf,
-	size_t count,
+	const off_t offset,
 	const uint64_t hdd_write_size,
 	const int hdd_flags)
 {
 	ssize_t ret;
 
-#if !defined(__sun__)
+#if defined(HAVE_FUTIMES)
 	if (hdd_flags & HDD_OPT_UTIMES)
-		(void)futimes(fd, NULL);
+		stress_hdd_utimes(fd);
+#else
+	UNEXPECTED
 #endif
-
+	errno = 0;
 	if (hdd_flags & HDD_OPT_IOVEC) {
+#if defined(HAVE_SYS_UIO_H)
 		struct iovec iov[HDD_IO_VEC_MAX];
 		size_t i;
 		uint8_t *data = buf;
@@ -177,24 +230,60 @@ static ssize_t stress_hdd_write(
 			iov[i].iov_base = (void *)data;
 			iov[i].iov_len = (size_t)sz;
 
-			buf += sz;
+			data += sz;
 		}
-		ret = writev(fd, iov, HDD_IO_VEC_MAX);
+
+		switch (stress_mwc8modn(3)) {
+#if defined(HAVE_PWRITEV2)
+		case 0:
+			ret = pwritev2(fd, iov, HDD_IO_VEC_MAX, offset, 0);
+			break;
+#endif
+#if defined(HAVE_PWRITEV)
+		case 1:
+			ret = pwritev(fd, iov, HDD_IO_VEC_MAX, offset);
+			break;
+#endif
+		default:
+			if (lseek(fd, offset, SEEK_SET) < 0) {
+				ret = -1;
+			} else {
+				ret = writev(fd, iov, HDD_IO_VEC_MAX);
+			}
+			break;
+		}
+#else
+		if (lseek(fd, offset, SEEK_SET) < 0) {
+			ret = -1;
+		} else {
+			ret = write(fd, buf, (size_t)hdd_write_size);
+		}
+#endif
 	} else {
-		ret = write(fd, buf, count);
+		if (lseek(fd, offset, SEEK_SET) < 0) {
+			ret = -1;
+		} else {
+			ret = write(fd, buf, (size_t)hdd_write_size);
+		}
 	}
 
-#if _BSD_SOURCE || _XOPEN_SOURCE || _POSIX_C_SOURCE >= 200112L
+#if defined(HAVE_FSYNC)
 	if (hdd_flags & HDD_OPT_FSYNC)
-		(void)fsync(fd);
+		(void)shim_fsync(fd);
+#else
+	UNEXPECTED
 #endif
-#if _POSIX_C_SOURCE >= 199309L || _XOPEN_SOURCE >= 500
+#if defined(HAVE_FDATASYNC)
 	if (hdd_flags & HDD_OPT_FDATASYNC)
-		(void)fdatasync(fd);
+		(void)shim_fdatasync(fd);
+#else
+	UNEXPECTED
 #endif
-#if NEED_GLIBC(2,14,0) && defined(__linux__)
+#if defined(HAVE_SYNCFS)
 	if (hdd_flags & HDD_OPT_SYNCFS)
 		(void)syncfs(fd);
+#else
+	UNEXPECTED
 #endif
 
 	return ret;
@@ -207,46 +296,187 @@ static ssize_t stress_hdd_write(
 static ssize_t stress_hdd_read(
 	const int fd,
 	uint8_t *buf,
-	size_t count,
-	const uint64_t hdd_write_size,
+	const off_t offset,
+	const uint64_t hdd_read_size,
 	const int hdd_flags)
 {
-#if !defined(__sun__)
+#if defined(HAVE_FUTIMES)
 	if (hdd_flags & HDD_OPT_UTIMES)
-		(void)futimes(fd, NULL);
+		stress_hdd_utimes(fd);
+#else
+	UNEXPECTED
 #endif
 
+	errno = 0;
 	if (hdd_flags & HDD_OPT_IOVEC) {
+#if defined(HAVE_SYS_UIO_H)
 		struct iovec iov[HDD_IO_VEC_MAX];
 		size_t i;
 		uint8_t *data = buf;
-		const uint64_t sz = hdd_write_size / HDD_IO_VEC_MAX;
+		const uint64_t sz = hdd_read_size / HDD_IO_VEC_MAX;
 
 		for (i = 0; i < HDD_IO_VEC_MAX; i++) {
 			iov[i].iov_base = (void *)data;
 			iov[i].iov_len = (size_t)sz;
 
-			buf += sz;
+			data += sz;
 		}
-		return readv(fd, iov, HDD_IO_VEC_MAX);
+		switch (stress_mwc8modn(3)) {
+#if defined(HAVE_PREADV2)
+		case 0:
+			return preadv2(fd, iov, HDD_IO_VEC_MAX, offset, 0);
+#endif
+#if defined(HAVE_PREADV)
+		case 1:
+			return preadv(fd, iov, HDD_IO_VEC_MAX, offset);
+#endif
+		default:
+			if (lseek(fd, offset, SEEK_SET) < 0)
+				return -1;
+			return readv(fd, iov, HDD_IO_VEC_MAX);
+		}
+#else
+		if (lseek(fd, offset, SEEK_SET) < 0)
+			return -1;
+		return read(fd, buf, (size_t)hdd_read_size);
+#endif
 	} else {
-		return read(fd, buf, count);
+		if (lseek(fd, offset, SEEK_SET) < 0)
+			return -1;
+		return read(fd, buf, (size_t)hdd_read_size);
 	}
 }
 
+/*
+ *  stress_hdd_invalid_read()
+ *	exercise invalid reads
+ */
+static void stress_hdd_invalid_read(const int fd, uint8_t *buf)
+{
+#if defined(HAVE_SYS_UIO_H)
+	struct iovec iov[HDD_IO_VEC_MAX];
+	size_t i;
+	uint8_t *data = buf;
+#endif
+	(void)fd;
+
+#if defined(HAVE_SYS_UIO_H)
+	for (i = 0; i < HDD_IO_VEC_MAX; i++) {
+		iov[i].iov_base = (void *)data;
+		iov[i].iov_len = (size_t)1;
+		data++;
+	}
+#endif
+
+#if defined(HAVE_SYS_UIO_H) &&	\
+    defined(HAVE_PREADV2)
+	/* invalid preadv2 fd */
+	VOID_RET(ssize_t, preadv2(-1, iov, HDD_IO_VEC_MAX, 0, 0));
+
+	/* invalid preadv2 offset, don't use -1 */
+	VOID_RET(ssize_t, preadv2(fd, iov, HDD_IO_VEC_MAX, -2, 0));
+
+	/* invalid preadv2 flags */
+	VOID_RET(ssize_t, preadv2(fd, iov, HDD_IO_VEC_MAX, 0, ~0));
+#else
+	UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UIO_H) &&	\
+    defined(HAVE_PREADV)
+	/* invalid preadv fd */
+	VOID_RET(ssize_t, preadv(-1, iov, HDD_IO_VEC_MAX, 0));
+
+	/* invalid preadv offset */
+	VOID_RET(ssize_t, preadv(fd, iov, HDD_IO_VEC_MAX, -1));
+#else
+	UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UIO_H)
+	/* invalid readv fd */
+	VOID_RET(ssize_t, readv(-1, iov, HDD_IO_VEC_MAX));
+#endif
+
+	/* invalid read fd */
+	VOID_RET(ssize_t, read(-1, buf, 1));
+}
 
 /*
- *  stress_hdd_opts
+ *  stress_hdd_invalid_write()
+ *	exercise invalid writess
+ */
+static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
+{
+#if defined(HAVE_SYS_UIO_H)
+	struct iovec iov[HDD_IO_VEC_MAX];
+	size_t i;
+	uint8_t *data = buf;
+#endif
+	(void)fd;
+
+#if defined(HAVE_SYS_UIO_H)
+	for (i = 0; i < HDD_IO_VEC_MAX; i++) {
+		iov[i].iov_base = (void *)data;
+		iov[i].iov_len = (size_t)1;
+		data++;
+	}
+#else
+	UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UIO_H) &&	\
+    defined(HAVE_PWRITEV2)
+	/* invalid pwritev2 fd */
+	VOID_RET(ssize_t, pwritev2(-1, iov, HDD_IO_VEC_MAX, 0, 0));
+
+	/* invalid pwritev2 offset, don't use -1 */
+	VOID_RET(ssize_t, pwritev2(fd, iov, HDD_IO_VEC_MAX, -2, 0));
+
+	/* invalid pwritev2 flags */
+	VOID_RET(ssize_t, pwritev2(fd, iov, HDD_IO_VEC_MAX, 0, ~0));
+#else
+	UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UIO_H) &&	\
+    defined(HAVE_PWRITEV)
+	/* invalid pwritev fd */
+	VOID_RET(ssize_t, pwritev(-1, iov, HDD_IO_VEC_MAX, 0));
+
+	/* invalid pwritev offset */
+	VOID_RET(ssize_t, pwritev(fd, iov, HDD_IO_VEC_MAX, -1));
+#else
+	UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UIO_H)
+	/* invalid writev fd */
+	VOID_RET(ssize_t, writev(-1, iov, HDD_IO_VEC_MAX));
+#else
+	UNEXPECTED
+#endif
+
+	/* invalid write fd */
+	VOID_RET(ssize_t, write(-1, buf, 1));
+}
+
+/*
+ *  stress_set_hdd_opts
  *	parse --hdd-opts option(s) list
  */
-int stress_hdd_opts(char *opts)
+static int stress_set_hdd_opts(const char *opts)
 {
-	char *str, *token;
+	char *str, *ptr, *token;
 	int hdd_flags = 0;
 	int hdd_oflags = 0;
 	bool opts_set = false;
 
-	for (str = opts; (token = strtok(str, ",")) != NULL; str = NULL) {
+	str = stress_const_optdup(opts);
+	if (!str)
+		return -1;
+
+	for (ptr = str; (token = strtok(ptr, ",")) != NULL; ptr = NULL) {
 		size_t i;
 		bool opt_ok = false;
 
@@ -266,6 +496,7 @@ int stress_hdd_opts(char *opts)
 							break;
 						}
 					}
+					free(str);
 					return -1;
 				}
 				hdd_flags  |= hdd_opts[i].flag;
@@ -280,13 +511,15 @@ int stress_hdd_opts(char *opts)
 				(void)fprintf(stderr, "%s %s",
 					i == 0 ? "" : ",", hdd_opts[i].opt);
 			(void)fprintf(stderr, "\n");
+			free(str);
 			return -1;
 		}
 	}
 
-	set_setting("hdd-flags", TYPE_ID_INT, &hdd_flags);
-	set_setting("hdd-oflags", TYPE_ID_INT, &hdd_oflags);
-	set_setting("hdd-opts-set", TYPE_ID_BOOL, &opts_set);
+	stress_set_setting("hdd-flags", TYPE_ID_INT, &hdd_flags);
+	stress_set_setting("hdd-oflags", TYPE_ID_INT, &hdd_oflags);
+	stress_set_setting("hdd-opts-set", TYPE_ID_BOOL, &opts_set);
+	free(str);
 
 	return 0;
 }
@@ -295,12 +528,12 @@ int stress_hdd_opts(char *opts)
  *  stress_hdd_advise()
  *	set posix_fadvise options
  */
-static int stress_hdd_advise(const args_t *args, const int fd, const int flags)
+static int stress_hdd_advise(const stress_args_t *args, const int fd, const int flags)
 {
-#if (defined(POSIX_FADV_SEQ) || defined(POSIX_FADV_RANDOM) || \
-    defined(POSIX_FADV_NOREUSE) || defined(POSIX_FADV_WILLNEED) || \
-    defined(POSIX_FADV_DONTNEED)) && !defined(__gnu_hurd__) && \
-    !defined(__minix__)
+#if (defined(POSIX_FADV_SEQ) || defined(POSIX_FADV_RANDOM) ||		\
+     defined(POSIX_FADV_NOREUSE) || defined(POSIX_FADV_WILLNEED) ||	\
+     defined(POSIX_FADV_DONTNEED)) && 					\
+     defined(HAVE_POSIX_FADVISE)
 	size_t i;
 
 	if (!(flags & HDD_OPT_FADV_MASK))
@@ -309,7 +542,8 @@ static int stress_hdd_advise(const args_t *args, const int fd, const int flags)
 	for (i = 0; i < SIZEOF_ARRAY(hdd_opts); i++) {
 		if (hdd_opts[i].flag & flags) {
 			if (posix_fadvise(fd, 0, 0, hdd_opts[i].advice) < 0) {
-				pr_fail_err("posix_fadvise");
+				pr_fail("%s: posix_fadvise failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
 				return -1;
 			}
 		}
@@ -321,15 +555,27 @@ static int stress_hdd_advise(const args_t *args, const int fd, const int flags)
 #endif
 	return 0;
 }
+
+/*
+ *  data_value()
+ *	generate 8 bit data value for offsets and instance # into a test file
+ */
+static inline uint8_t data_value(const uint64_t i, uint64_t j, const stress_args_t *args)
+{
+	register uint8_t v = (uint8_t)(((i + j) >> 9) + i + j + args->instance);
+
+	return v;
+}
+
 /*
  *  stress_hdd
  *	stress I/O via writes
  */
-int stress_hdd(const args_t *args)
+static int stress_hdd(const stress_args_t *args)
 {
 	uint8_t *buf = NULL;
-	uint8_t *alloc_buf;
-	uint64_t i, min_size, remainder;
+	void *alloc_buf;
+	uint64_t i, min_size, size_remainder;
 	int rc = EXIT_FAILURE;
 	ssize_t ret;
 	char filename[PATH_MAX];
@@ -340,16 +586,16 @@ int stress_hdd(const args_t *args)
 	int flags, fadvise_flags;
 	bool opts_set = false;
 
-	(void)get_setting("hdd-flags", &hdd_flags);
-	(void)get_setting("hdd-oflags", &hdd_oflags);
-	(void)get_setting("hdd-opts-set", &opts_set);
+	(void)stress_get_setting("hdd-flags", &hdd_flags);
+	(void)stress_get_setting("hdd-oflags", &hdd_oflags);
+	(void)stress_get_setting("hdd-opts-set", &opts_set);
 
 	flags = O_CREAT | O_RDWR | O_TRUNC | hdd_oflags;
 	fadvise_flags = hdd_flags & HDD_OPT_FADV_MASK;
 
-	if (!get_setting("hdd-bytes", &hdd_bytes)) {
+	if (!stress_get_setting("hdd-bytes", &hdd_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			hdd_bytes = MAX_HDD_BYTES;
+			hdd_bytes = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			hdd_bytes = MIN_HDD_BYTES;
 	}
@@ -358,7 +604,7 @@ int stress_hdd(const args_t *args)
 	if (hdd_bytes < MIN_HDD_WRITE_SIZE)
 		hdd_bytes = MIN_HDD_WRITE_SIZE;
 
-	if (!get_setting("hdd-write-size", &hdd_write_size)) {
+	if (!stress_get_setting("hdd-write-size", &hdd_write_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			hdd_write_size = MAX_HDD_WRITE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
@@ -380,9 +626,9 @@ int stress_hdd(const args_t *args)
 	}
 
 	/* Ensure we get same sized iovec I/O sizes */
-	remainder = hdd_write_size % HDD_IO_VEC_MAX;
-	if ((hdd_flags & HDD_OPT_IOVEC) && (remainder != 0)) {
-		hdd_write_size += HDD_IO_VEC_MAX - remainder;
+	size_remainder = hdd_write_size % HDD_IO_VEC_MAX;
+	if ((hdd_flags & HDD_OPT_IOVEC) && (size_remainder != 0)) {
+		hdd_write_size += HDD_IO_VEC_MAX - size_remainder;
 		pr_inf("%s: increasing read/write size to %"
 			PRIu64 " bytes in iovec mode\n",
 			args->name, hdd_write_size);
@@ -399,7 +645,7 @@ int stress_hdd(const args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
-		return exit_status(-ret);
+		return stress_exit_status((int)-ret);
 
 	/* Must have some write option */
 	if ((hdd_flags & HDD_OPT_WR_MASK) == 0)
@@ -408,7 +654,16 @@ int stress_hdd(const args_t *args)
 	if ((hdd_flags & HDD_OPT_RD_MASK) == 0)
 		hdd_flags |= HDD_OPT_RD_SEQ;
 
-#if defined(__sun__)
+#if defined(HAVE_POSIX_MEMALIGN)
+	ret = posix_memalign((void **)&alloc_buf, BUF_ALIGNMENT, (size_t)hdd_write_size);
+	if (ret || !alloc_buf) {
+		rc = stress_exit_status(errno);
+		pr_err("%s: cannot allocate buffer\n", args->name);
+		(void)stress_temp_dir_rm_args(args);
+		return rc;
+	}
+	buf = alloc_buf;
+#else
 	/* Work around lack of posix_memalign */
 	alloc_buf = malloc((size_t)hdd_write_size + BUF_ALIGNMENT);
 	if (!alloc_buf) {
@@ -416,26 +671,19 @@ int stress_hdd(const args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return rc;
 	}
-	buf = (uint8_t *)align_address(alloc_buf, BUF_ALIGNMENT);
-#else
-	ret = posix_memalign((void **)&alloc_buf, BUF_ALIGNMENT, (size_t)hdd_write_size);
-	if (ret || !alloc_buf) {
-		rc = exit_status(errno);
-		pr_err("%s: cannot allocate buffer\n", args->name);
-		(void)stress_temp_dir_rm_args(args);
-		return rc;
-	}
-	buf = alloc_buf;
+	buf = (uint8_t *)stress_align_address(alloc_buf, BUF_ALIGNMENT);
 #endif
-
-	stress_strnrnd((char *)buf, hdd_write_size);
-
+	(void)memset(buf, stress_mwc8(), hdd_write_size);
 	(void)stress_temp_filename_args(args,
-		filename, sizeof(filename), mwc32());
+		filename, sizeof(filename), stress_mwc32());
+
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	do {
 		int fd;
 		struct stat statbuf;
 		uint64_t hdd_read_size;
+		const char *fs_type;
 
 		/*
 		 * aggressive option with no other option enables
@@ -444,7 +692,7 @@ int stress_hdd(const args_t *args)
 		if (!opts_set && (g_opt_flags & OPT_FLAGS_AGGRESSIVE)) {
 			opt_index = (opt_index + 1) % SIZEOF_ARRAY(hdd_opts);
 
-			hdd_flags  = hdd_opts[opt_index].flag;
+			hdd_flags = hdd_opts[opt_index].flag;
 			hdd_oflags = hdd_opts[opt_index].oflag;
 			if ((hdd_flags & HDD_OPT_WR_MASK) == 0)
 				hdd_flags |= HDD_OPT_WR_SEQ;
@@ -452,47 +700,97 @@ int stress_hdd(const args_t *args)
 				hdd_flags |= HDD_OPT_RD_SEQ;
 		}
 
-		(void)umask(0077);
 		if ((fd = open(filename, flags, S_IRUSR | S_IWUSR)) < 0) {
 			if ((errno == ENOSPC) || (errno == ENOMEM))
 				continue;	/* Retry */
-			pr_fail_err("open");
+			pr_fail("%s: open %s failed, errno=%d (%s)\n",
+				args->name, filename, errno, strerror(errno));
+			/*
+			 *  Unlink is necessary as Linux can leave stale files
+			 *  on a O_DIRECT open failure if the file system does
+			 *  not support O_DIRECT:
+			 *  https://bugzilla.kernel.org/show_bug.cgi?id=213041
+			 */
+			(void)shim_unlink(filename);
 			goto finish;
 		}
-		if (ftruncate(fd, (off_t)0) < 0) {
-			pr_fail_err("ftruncate");
+
+		fs_type = stress_fs_type(filename);
+
+#if defined(F_SET_FILE_RW_HINT) &&	\
+    defined(RWH_WRITE_LIFE_SHORT)
+		{
+			uint64_t hint = RWH_WRITE_LIFE_SHORT;
+			VOID_RET(int, fcntl(fd, F_SET_FILE_RW_HINT, &hint));
+		}
+#endif
+
+		/* Exercise ftruncate or truncate */
+		if (stress_mwc1()) {
+			if (ftruncate(fd, (off_t)0) < 0) {
+				pr_fail("%s: ftruncate failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
+				(void)close(fd);
+				goto finish;
+			}
+			/* Exercise invalid ftruncate size, EINVAL */
+			VOID_RET(int, ftruncate(fd, (off_t)-1));
+
+			/* Exercise invalid fd, EBADF */
+			VOID_RET(int, ftruncate(-1, (off_t)0));
+		} else {
+			if (truncate(filename, (off_t)0) < 0) {
+				pr_fail("%s: truncate failed, errno=%d (%s)%s\n",
+					args->name, errno, strerror(errno), fs_type);
+				(void)close(fd);
+				goto finish;
+			}
+			/* Exercise invalid truncate size, EINVAL */
+			VOID_RET(int, truncate(filename, (off_t)-1));
+
+			/* Exercise invalid path, ENOENT */
+			VOID_RET(int, truncate("", (off_t)0));
+		}
+		(void)shim_unlink(filename);
+
+		if (!keep_stressing(args)) {
 			(void)close(fd);
-			goto finish;
+			goto yielded;
 		}
-		(void)unlink(filename);
 
 		if (stress_hdd_advise(args, fd, fadvise_flags) < 0) {
 			(void)close(fd);
 			goto finish;
 		}
 
+		/* exercise invalid I/O calls */
+		stress_hdd_invalid_write(fd, buf);
+		stress_hdd_invalid_read(fd, buf);
+
 		/* Random Write */
 		if (hdd_flags & HDD_OPT_WR_RND) {
+			uint32_t w, z;
+
+			w = stress_mwc32();
+			z = stress_mwc32();
+
+			stress_mwc_set_seed(w, z);
+
 			for (i = 0; i < hdd_bytes; i += hdd_write_size) {
 				size_t j;
-
-				off_t offset = (i == 0) ?
-					hdd_bytes :
-					(mwc64() % hdd_bytes) & ~511;
-
-				if (lseek(fd, offset, SEEK_SET) < 0) {
-					pr_fail_err("lseek");
-					(void)close(fd);
-					goto finish;
-				}
+				uint64_t offset = (i == 0) ?
+					hdd_bytes : stress_mwc64modn(hdd_bytes) & ~511UL;
 rnd_wr_retry:
-				if (!keep_stressing())
-					break;
+				if (!keep_stressing(args)) {
+					(void)close(fd);
+					goto yielded;
+				}
 
-				for (j = 0; j < hdd_write_size; j++)
-					buf[j] = (offset + j) & 0xff;
+				for (j = 0; j < hdd_write_size; j++) {
+					buf[j] = data_value(offset, j, args);
+				}
 
-				ret = stress_hdd_write(fd, buf, (size_t)hdd_write_size,
+				ret = stress_hdd_write(fd, buf, (off_t)offset,
 					hdd_write_size, hdd_flags);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
@@ -500,7 +798,8 @@ rnd_wr_retry:
 					if (errno == ENOSPC)
 						break;
 					if (errno) {
-						pr_fail_err("write");
+						pr_fail("%s: write failed, errno=%d (%s)%s\n",
+							args->name, errno, strerror(errno), fs_type);
 						(void)close(fd);
 						goto finish;
 					}
@@ -508,18 +807,22 @@ rnd_wr_retry:
 				}
 				inc_counter(args);
 			}
+
+			stress_mwc_set_seed(w, z);
 		}
 		/* Sequential Write */
 		if (hdd_flags & HDD_OPT_WR_SEQ) {
 			for (i = 0; i < hdd_bytes; i += hdd_write_size) {
 				size_t j;
 seq_wr_retry:
-				if (!keep_stressing())
-					break;
+				if (!keep_stressing(args)) {
+					(void)close(fd);
+					goto yielded;
+				}
 
-				for (j = 0; j < hdd_write_size; j += 512)
-					buf[j] = (i + j) & 0xff;
-				ret = stress_hdd_write(fd, buf, (size_t)hdd_write_size,
+				for (j = 0; j < hdd_write_size; j++)
+					buf[j] = data_value(i, j, args);
+				ret = stress_hdd_write(fd, buf, (off_t)i,
 					hdd_write_size, hdd_flags);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
@@ -527,7 +830,8 @@ seq_wr_retry:
 					if (errno == ENOSPC)
 						break;
 					if (errno) {
-						pr_fail_err("write");
+						pr_fail("%s: write failed, errno=%d (%s)%s\n",
+							args->name, errno, strerror(errno), fs_type);
 						(void)close(fd);
 						goto finish;
 					}
@@ -538,56 +842,64 @@ seq_wr_retry:
 		}
 
 		if (fstat(fd, &statbuf) < 0) {
-			pr_fail_err("fstat");
+			pr_fail("%s: fstat failed, errno=%d (%s)%s\n",
+				args->name, errno, strerror(errno), fs_type);
 			(void)close(fd);
 			continue;
 		}
 		/* Round to write size to get no partial reads */
 		hdd_read_size = (uint64_t)statbuf.st_size -
-			(statbuf.st_size % hdd_write_size);
+			((uint64_t)statbuf.st_size % hdd_write_size);
 
 		/* Sequential Read */
 		if (hdd_flags & HDD_OPT_RD_SEQ) {
 			uint64_t misreads = 0;
 			uint64_t baddata = 0;
 
-			if (lseek(fd, 0, SEEK_SET) < 0) {
-				pr_fail_err("lseek");
-				(void)close(fd);
-				goto finish;
-			}
 			for (i = 0; i < hdd_read_size; i += hdd_write_size) {
 seq_rd_retry:
-				if (!keep_stressing())
-					break;
-
-				ret = stress_hdd_read(fd, buf, (size_t)hdd_write_size,
+				if (!keep_stressing(args)) {
+					(void)close(fd);
+					goto yielded;
+				}
+				ret = stress_hdd_read(fd, buf, (off_t)i,
 					hdd_write_size, hdd_flags);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						goto seq_rd_retry;
+					if (errno == ENOSPC)	/* e.g. on vfat */
+						continue;
 					if (errno) {
-						pr_fail_err("read");
+						pr_fail("%s: read failed, errno=%d (%s)%s\n",
+							args->name, errno, strerror(errno), fs_type);
 						(void)close(fd);
 						goto finish;
 					}
 					continue;
 				}
-				if (ret != (ssize_t)hdd_write_size)
+				if (ret != (ssize_t)hdd_write_size) {
 					misreads++;
+				}
 
 				if (g_opt_flags & OPT_FLAGS_VERIFY) {
-					size_t j;
+					if (hdd_flags & HDD_OPT_WR_SEQ) {
+						size_t j;
 
-					for (j = 0; j < hdd_write_size; j += 512) {
-						uint8_t v = (i + j) & 0xff;
-						if (hdd_flags & HDD_OPT_WR_SEQ) {
-							/* Write seq has written to all of the file, so it should always be OK */
-							if (buf[0] != v)
+						/* Write seq has written to all of the file, so it should always be OK */
+						for (j = 0; j < (size_t)ret; j++) {
+							const uint8_t v = data_value(i, j, args);
+
+							if (buf[j] != v)
 								baddata++;
-						} else {
-							/* Write rnd has written to some of the file, so data either zero or OK */
-							if (buf[0] != 0 && buf[0] != v)
+						}
+					} else {
+						size_t j;
+
+						/* Write rnd has written to some of the file, so data either zero or OK */
+						for (j = 0; j < (size_t)ret; j++) {
+							const uint8_t v = data_value(i, j, args);
+
+							if ((buf[j] != 0) && (buf[j] != v))
 								baddata++;
 						}
 					}
@@ -599,8 +911,8 @@ seq_rd_retry:
 					" incomplete sequential reads\n",
 					args->name, misreads);
 			if (baddata)
-				pr_fail("%s: incorrect data found %"
-					PRIu64 " times\n", args->name, baddata);
+				pr_fail("%s: incorrect data found %" PRIu64 " times\n",
+					args->name, baddata);
 		}
 		/* Random Read */
 		if (hdd_flags & HDD_OPT_RD_RND) {
@@ -608,24 +920,30 @@ seq_rd_retry:
 			uint64_t baddata = 0;
 
 			for (i = 0; i < hdd_read_size; i += hdd_write_size) {
-				off_t offset = (hdd_bytes > hdd_write_size) ?
-					(mwc64() % (hdd_bytes - hdd_write_size)) & ~511 : 0;
+				size_t offset = (hdd_bytes > hdd_write_size) ?
+					stress_mwc64modn(hdd_bytes - hdd_write_size) & ~511UL : 0;
 
-				if (lseek(fd, offset, SEEK_SET) < 0) {
-					pr_fail_err("lseek");
+				if (lseek(fd, (off_t)offset, SEEK_SET) < 0) {
+					pr_fail("%s: lseek failed, errno=%d (%s)%s\n",
+						args->name, errno, strerror(errno), fs_type);
 					(void)close(fd);
 					goto finish;
 				}
 rnd_rd_retry:
-				if (!keep_stressing())
-					break;
-				ret = stress_hdd_read(fd, buf, (size_t)hdd_write_size,
+				if (!keep_stressing(args)) {
+					(void)close(fd);
+					goto yielded;
+				}
+				ret = stress_hdd_read(fd, buf, (off_t)offset,
 					hdd_write_size, hdd_flags);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						goto rnd_rd_retry;
+					if (errno == ENOSPC)	/* e.g. on vfat */
+						continue;
 					if (errno) {
-						pr_fail_err("read");
+						pr_fail("%s: read failed, errno=%d (%s)%s\n",
+							args->name, errno, strerror(errno), fs_type);
 						(void)close(fd);
 						goto finish;
 					}
@@ -637,15 +955,15 @@ rnd_rd_retry:
 				if (g_opt_flags & OPT_FLAGS_VERIFY) {
 					size_t j;
 
-					for (j = 0; j < hdd_write_size; j += 512) {
-						uint8_t v = (i + j) & 0xff;
+					for (j = 0; j < (size_t)ret; j++) {
+						uint8_t v = data_value(offset, j, args);
 						if (hdd_flags & HDD_OPT_WR_SEQ) {
 							/* Write seq has written to all of the file, so it should always be OK */
-							if (buf[0] != v)
+							if (buf[j] != v)
 								baddata++;
 						} else {
 							/* Write rnd has written to some of the file, so data either zero or OK */
-							if (buf[0] != 0 && buf[0] != v)
+							if (buf[j] != 0 && buf[j] != v)
 								baddata++;
 						}
 					}
@@ -656,14 +974,35 @@ rnd_rd_retry:
 				pr_dbg("%s: %" PRIu64
 					" incomplete random reads\n",
 					args->name, misreads);
+			if (baddata)
+				pr_fail("%s: incorrect data found %" PRIu64 " times\n",
+					args->name, baddata);
 		}
+
 		(void)close(fd);
+	} while (keep_stressing(args));
 
-	} while (keep_stressing());
-
+yielded:
 	rc = EXIT_SUCCESS;
 finish:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	free(alloc_buf);
 	(void)stress_temp_dir_rm_args(args);
 	return rc;
 }
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_hdd_bytes,	stress_set_hdd_bytes },
+	{ OPT_hdd_opts,		stress_set_hdd_opts },
+	{ OPT_hdd_write_size,	stress_set_hdd_write_size },
+	{ 0,			NULL },
+};
+
+stressor_info_t stress_hdd_info = {
+	.stressor = stress_hdd,
+	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
+	.help = help
+};
