@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King.
+ * Copyright (C) 2021-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -165,7 +165,7 @@ static int stress_kill(const stress_args_t *args)
 	}
 
 	rate = (duration > 0.0) ? count / duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "kill calls per sec", rate);
+	stress_metrics_set(args, 0, "kill calls per sec", rate);
 
 	return EXIT_SUCCESS;
 }
