@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -253,7 +253,7 @@ reap:
 	pr_dbg("%s: %" PRIu64 " lease sigio interrupts caught\n", args->name, lease_sigio);
 	dt = t2 - t1;
 	if (dt > 0.0) {
-		stress_misc_stats_set(args->misc_stats, 0, "lease sigio interrupts per sec",
+		stress_metrics_set(args, 0, "lease sigio interrupts per sec",
 			(double)lease_sigio / dt);
 	}
 
