@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -37,7 +32,6 @@ static const stress_help_t help[] = {
     defined(F_UNLCK)
 
 #define LOCK_FILE_SIZE	(1024 * 1024)
-#define LOCK_SIZE	(8)
 #define LOCK_MAX	(1024)
 
 typedef struct stress_lockofd_info {
@@ -181,7 +175,7 @@ static int stress_lockofd_contention(
 				return -1;
 
 		len = (stress_mwc16() + 1) & 0xfff;
-		offset = (off_t)stress_mwc64() % (LOCK_FILE_SIZE - len);
+		offset = (off_t)stress_mwc64modn((uint64_t)(LOCK_FILE_SIZE - len));
 
 		f.l_type = F_WRLCK;
 		f.l_whence = SEEK_SET;
@@ -197,7 +191,7 @@ static int stress_lockofd_contention(
 
 		lockofd_info = stress_lockofd_info_new();
 		if (!lockofd_info) {
-			pr_fail("%s: calloc failed, out of memory\n", args->name);
+			pr_err("%s: calloc failed, out of memory\n", args->name);
 			return -1;
 		}
 		lockofd_info->offset = offset;
@@ -232,8 +226,8 @@ static int stress_lockofd(const stress_args_t *args)
 	(void)stress_temp_dir_args(args, pathname, sizeof(pathname));
 	if (mkdir(pathname, S_IRWXU) < 0) {
 		if (errno != EEXIST) {
-			ret = exit_status(errno);
-			pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
+			ret = stress_exit_status(errno);
+			pr_err("%s: mkdir %s failed, errno=%d (%s)\n",
 				args->name, pathname, errno, strerror(errno));
 			return ret;
 		}
@@ -248,15 +242,15 @@ static int stress_lockofd(const stress_args_t *args)
 		filename, sizeof(filename), stress_mwc32());
 
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
-		ret = exit_status(errno);
-		pr_fail("%s: open %s failed, errno=%d (%s)\n",
+		ret = stress_exit_status(errno);
+		pr_err("%s: open %s failed, errno=%d (%s)\n",
 			args->name, pathname, errno, strerror(errno));
-		(void)rmdir(pathname);
+		(void)shim_rmdir(pathname);
 		return ret;
 	}
 
 	if (lseek(fd, 0, SEEK_SET) < 0) {
-		pr_fail("%s: lseek failed, errno=%d (%s)\n",
+		pr_err("%s: lseek failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
@@ -270,8 +264,8 @@ redo:
 		if ((rc < 0) || (rc != sizeof(buffer))) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo;
-			ret = exit_status(errno);
-			pr_fail("%s: write failed, errno=%d (%s)\n",
+			ret = stress_exit_status(errno);
+			pr_err("%s: write failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto tidy;
 		}
@@ -285,12 +279,11 @@ again:
 			goto again;
 		if (!keep_stressing(args))
 			goto tidy;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
 	if (cpid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -299,7 +292,6 @@ again:
 		stress_lockofd_info_free();
 		_exit(EXIT_SUCCESS);
 	}
-	(void)setpgid(cpid, g_pgrp);
 
 	if (stress_lockofd_contention(args, fd) == 0)
 		ret = EXIT_SUCCESS;
@@ -315,8 +307,8 @@ tidy:
 	stress_lockofd_info_free();
 
 	(void)close(fd);
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return ret;
 }
@@ -324,12 +316,14 @@ tidy:
 stressor_info_t stress_lockofd_info = {
 	.stressor = stress_lockofd,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_lockofd_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without fcntl() F_OFD_GETLK, F_OFD_SETLK, F_OFD_SETLKW, F_WRLCK or F_UNLCK commands"
 };
 #endif
