@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King.
+ * Copyright (C) 2021-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -89,11 +89,24 @@ static const unsigned int flags[] = {
 #endif
 };
 
+/*
+ *  uint64_ptr_offset()
+ *	add offset to 64 bit ptr
+ */
+static inline const uint64_t *uint64_ptr_offset(const uint64_t *ptr, const size_t offset)
+{
+	return (uint64_t *)((uintptr_t)ptr + offset);
+}
+
+/*
+ *  stress_memfd_fill_pages()
+ *	fill pages with random uin64_t values
+ */
 static inline void TARGET_CLONES stress_memfd_fill_pages(void *ptr, const size_t size)
 {
-	uint64_t *u64ptr = (uint64_t *)ptr;
-	uint64_t *u64end = (uint64_t *)((uintptr_t)ptr + size);
-	uint64_t v = stress_mwc64();
+	register uint64_t *u64ptr = (uint64_t *)ptr;
+	register const uint64_t *u64end = uint64_ptr_offset(ptr, size);
+	register uint64_t v = stress_mwc64();
 
 	while (u64ptr < u64end) {
 		*u64ptr++ = v;
@@ -132,6 +145,27 @@ static inline void TARGET_CLONES stress_memfd_fill_pages(void *ptr, const size_t
 	}
 }
 
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_PAGEOUT)
+/*
+ *  stress_memfd_check()
+ *	check if buffer buf contains uint64_t values val, return true if OK, false if not
+ */
+static bool stress_memfd_check(
+	const uint64_t val,
+	const uint64_t *buf,
+	const size_t size)
+{
+	const uint64_t *ptr, *end_ptr = uint64_ptr_offset(buf, size);
+
+	for (ptr = buf; ptr < end_ptr; ptr++) {
+		if (*ptr != val)
+			return false;
+	}
+	return true;
+}
+#endif
+
 /*
  *  Create allocations using memfd_create, ftruncate and mmap
  */
@@ -146,6 +180,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 	size_t memfd_bytes = DEFAULT_MEMFD_BYTES;
 	uint32_t memfd_fds = DEFAULT_MEMFD_FDS;
 	int mmap_flags = MAP_FILE | MAP_SHARED;
+	double duration = 0.0, count = 0.0, rate;
 
 #if defined(MAP_POPULATE)
 	mmap_flags |= MAP_POPULATE;
@@ -187,6 +222,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 
 	do {
 		char filename[PATH_MAX];
+		double t;
 
 		for (i = 0; i < memfd_fds; i++) {
 			fds[i] = -1;
@@ -201,25 +237,30 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			(void)snprintf(filename, sizeof(filename),
 				"memfd-%" PRIdMAX "-%" PRIu64,
 				(intmax_t)args->pid, i);
+
+			t = stress_time_now();
 			fds[i] = shim_memfd_create(filename, 0);
-			if (fds[i] < 0) {
+			if (fds[i] >= 0) {
+				duration += stress_time_now() - t;
+				count += 1.0;
+			} else {
 				switch (errno) {
 				case EMFILE:
 				case ENFILE:
 					break;
 				case ENOMEM:
-					goto clean;
+					goto memfd_unmap;
 				case ENOSYS:
 				case EFAULT:
 				default:
 					pr_fail("%s: memfd_create failed: errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					keep_stressing_set_flag(false);
-					goto clean;
+					goto memfd_unmap;
 				}
 			}
 			if (!keep_stressing_flag())
-				goto clean;
+				goto memfd_unmap;
 		}
 
 		for (i = 0; i < memfd_fds; i++) {
@@ -261,7 +302,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			/*
 			 *  ..and punch a hole
 			 */
-			whence = (off_t)((stress_mwc32() % size) & ~(page_size - 1));
+			whence = (off_t)(stress_mwc32modn(size) & ~(page_size - 1));
 			VOID_RET(ssize_t, shim_fallocate(fds[i], FALLOC_FL_PUNCH_HOLE |
 				FALLOC_FL_KEEP_SIZE, whence, (off_t)page_size));
 #endif
@@ -273,7 +314,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			VOID_RET(ssize_t, shim_fallocate(fds[i], 0, (off_t)size, 0));
 
 			if (!keep_stressing_flag())
-				goto clean;
+				goto memfd_unmap;
 		}
 
 		for (i = 0; i < memfd_fds; i++) {
@@ -315,18 +356,61 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			}
 #endif
 			if (!keep_stressing_flag())
-				goto clean;
+				goto memfd_unmap;
 		}
-clean:
+
+memfd_unmap:
 		for (i = 0; i < memfd_fds; i++) {
 			if (maps[i] != MAP_FAILED)
 				(void)munmap(maps[i], size);
+		}
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_PAGEOUT)
+		/*
+		 *  Check for zap_pte bug, see Linux commit
+		 *  5abfd71d936a8aefd9f9ccd299dea7a164a5d455
+		 */
+		for (i = 0; keep_stressing_flag() && (i < memfd_fds); i++) {
+			uint64_t *buf, *ptr;
+			const uint64_t *end_ptr;
+			uint64_t val = stress_mwc64();
+			const ssize_t test_size = page_size << 1;
+
+			if (ftruncate(fds[i], (off_t)test_size) < 0)
+				continue;
+			buf = mmap(NULL, test_size, PROT_READ | PROT_WRITE,
+					MAP_PRIVATE, fds[i], 0);
+			if (buf == MAP_FAILED)
+				continue;
+			end_ptr = uint64_ptr_offset(buf, test_size);
+			for (ptr = buf; ptr < end_ptr; ptr++)
+				*ptr = val;
+
+			if (madvise(buf, test_size, MADV_PAGEOUT) < 0)
+				goto buf_unmap;
+			if (ftruncate(fds[i], (off_t)page_size) < 0)
+				goto buf_unmap;
+			if (ftruncate(fds[i], (off_t)test_size) < 0)
+				goto buf_unmap;
+			if (!stress_memfd_check(val, buf, page_size))
+				pr_fail("%s: unexpected memfd %d data mismatch in first page\n",
+					args->name, fds[i]);
+			if (!stress_memfd_check(0ULL, uint64_ptr_offset(buf, page_size), page_size))
+				pr_fail("%s: unexpected memfd %d data mismatch in zero'd second page\n",
+					args->name, fds[i]);
+buf_unmap:
+			(void)munmap((void *)buf, test_size);
+			VOID_RET(int, ftruncate(fds[i], 0));
+		}
+#endif
+
+		for (i = 0; i < memfd_fds; i++) {
 			if (fds[i] >= 0)
 				(void)close(fds[i]);
 		}
 
 		/* Exercise illegal memfd name */
-		stress_strnrnd(filename, sizeof(filename));
+		stress_rndstr(filename, sizeof(filename));
 		fd = shim_memfd_create(filename, 0);
 		if (fd >= 0)
 			(void)close(fd);
@@ -371,15 +455,22 @@ clean:
 			(void)snprintf(filename, sizeof(filename),
 				"memfd-%" PRIdMAX"-%" PRIu64,
 				(intmax_t)args->pid, i);
+			t = stress_time_now();
 			fd = shim_memfd_create(filename, flags[i]);
-			if (fd >= 0)
+			if (fd >= 0) {
+				duration += stress_time_now() - t;
+				count += 1.0;
 				(void)close(fd);
+			}
 		}
 		inc_counter(args);
 	} while (keep_stressing(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	rate = (count > 0.0) ? duration / count : 0.0;
+	stress_metrics_set(args, 0, "nanosecs per memfd_create call", rate * STRESS_DBL_NANOSECOND);
+
 	free(maps);
 	free(fds);
 
