@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -85,6 +85,8 @@ static int stress_mergesort(const stress_args_t *args)
 	size_t n, i;
 	struct sigaction old_action;
 	int ret;
+	double rate;
+	NOCLOBBER double duration = 0.0, count = 0.0, sorted = 0.0;
 
 	if (!stress_get_setting("mergesort-size", &mergesort_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -117,18 +119,25 @@ static int stress_mergesort(const stress_args_t *args)
 		goto tidy;
 	}
 
-	/* This is expensive, do it once */
-	for (ptr = data, i = 0; i < n; i++)
-		*ptr++ = (int32_t)stress_mwc32();
-
+	stress_sort_data_int32_init(data, n);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
+		double t;
+
+		stress_sort_data_int32_shuffle(data, n);
+
+		stress_sort_compare_reset();
+		t = stress_time_now();
 		/* Sort "random" data */
 		if (mergesort(data, n, sizeof(*data), stress_sort_cmp_int32) < 0) {
 			pr_fail("%s: mergesort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else {
+			duration += stress_time_now() - t;
+			count += (double)stress_sort_compare_get();
+			sorted += (double)n;
+
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
 					if (*ptr > *(ptr + 1)) {
@@ -144,10 +153,16 @@ static int stress_mergesort(const stress_args_t *args)
 			break;
 
 		/* Reverse sort */
+		stress_sort_compare_reset();
+		t = stress_time_now();
 		if (mergesort(data, n, sizeof(*data), stress_sort_cmp_rev_int32) < 0) {
 			pr_fail("%s: reversed mergesort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else {
+			duration += stress_time_now() - t;
+			count += (double)stress_sort_compare_get();
+			sorted += (double)n;
+
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
 					if (*ptr < *(ptr + 1)) {
@@ -161,24 +176,38 @@ static int stress_mergesort(const stress_args_t *args)
 		}
 		if (!keep_stressing_flag())
 			break;
-		/* And re-order by byte compare */
+		/* And re-order by uint64 compare */
+		stress_sort_compare_reset();
+		t = stress_time_now();
 		if (mergesort(data, n / 2, sizeof(int64_t), stress_sort_cmp_int64) < 0) {
 			pr_fail("%s: mergesort failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
+		} else {
+			duration += stress_time_now() - t;
+			count += (double)stress_sort_compare_get();
+			sorted += (double)n;
 		}
 
 		/* Reverse sort this again */
+		stress_sort_compare_reset();
+		t = stress_time_now();
 		if (mergesort(data, n, sizeof(*data), stress_sort_cmp_rev_int32) < 0) {
 			pr_fail("%s: reversed mergesort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
-		}
-		if (g_opt_flags & OPT_FLAGS_VERIFY) {
-			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr < *(ptr + 1)) {
-					pr_fail("%s: reverse sort "
-						"error detected, incorrect "
-						"ordering found\n", args->name);
-					break;
+		} else {
+			duration += stress_time_now() - t;
+			count += (double)stress_sort_compare_get();
+			sorted += (double)n;
+
+			if (g_opt_flags & OPT_FLAGS_VERIFY) {
+
+				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
+					if (*ptr < *(ptr + 1)) {
+						pr_fail("%s: reverse sort "
+							"error detected, incorrect "
+							"ordering found\n", args->name);
+						break;
+					}
 				}
 			}
 		}
@@ -192,6 +221,9 @@ static int stress_mergesort(const stress_args_t *args)
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	rate = (duration > 0.0) ? count / duration : 0.0;
+	stress_metrics_set(args, 0, "mergesort comparisons per sec", rate);
+	stress_metrics_set(args, 1, "mergesort comparisons per item", count / sorted);
 
 	free(data);
 
