@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2021-2022 Colin Ian King
+ * Copyright (C) 2021-2023 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -379,8 +379,8 @@ static int stress_rdrand(const stress_args_t *args)
 		c = get_counter(args);
 		million_bits = ((double)c * 64.0 * 256.0) * ONE_MILLIONTH;
 		rate = (duration > 0.0) ? million_bits / duration : 0.0;
-		stress_misc_stats_set(args->misc_stats, 0, "million random bits read", million_bits);
-		stress_misc_stats_set(args->misc_stats, 1, "million random bits per sec", rate);
+		stress_metrics_set(args, 0, "million random bits read", million_bits);
+		stress_metrics_set(args, 1, "million random bits per sec", rate);
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
