@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +41,6 @@
 #define REVIO_OPT_O_DSYNC	(0x00020000)
 #define REVIO_OPT_O_DIRECT	(0x00040000)
 #define REVIO_OPT_O_NOATIME	(0x00080000)
-#define REVIO_OPT_O_MASK	(0x000f0000)
 
 /* Other modes */
 #define REVIO_OPT_UTIMES	(0x00100000)
@@ -324,12 +323,12 @@ static int stress_revio(const stress_args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
-		return exit_status((int)-ret);
+		return stress_exit_status((int)-ret);
 
 #if defined(HAVE_POSIX_MEMALIGN)
 	ret = posix_memalign((void **)&alloc_buf, BUF_ALIGNMENT, (size_t)DEFAULT_REVIO_WRITE_SIZE);
 	if (ret || !alloc_buf) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_err("%s: cannot allocate buffer\n", args->name);
 		(void)stress_temp_dir_rm_args(args);
 		return rc;
@@ -346,7 +345,7 @@ static int stress_revio(const stress_args_t *args)
 	buf = (uint8_t *)stress_align_address(alloc_buf, BUF_ALIGNMENT);
 #endif
 
-	stress_strnrnd((char *)buf, DEFAULT_REVIO_WRITE_SIZE);
+	stress_rndbuf(buf, DEFAULT_REVIO_WRITE_SIZE);
 
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
