@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -87,7 +87,7 @@ static void MLOCKED_TEXT stress_sigio_handler(int signum)
 static int stress_sigio(const stress_args_t *args)
 {
 	int ret, rc = EXIT_FAILURE, fds[2], status, flags = -1;
-	double t_start, t_delta;
+	double t_start, t_delta, rate;
 
 	rd_fd = -1;
 	sigio_args = args;
@@ -190,10 +190,8 @@ again:
 	} while (keep_stressing(args));
 
 	t_delta = stress_time_now() - t_start;
-
-	if (t_delta > 0.0) 
-		pr_inf("%s: %.2f SIGIO signals/sec\n",
-			args->name, (double)async_sigs / t_delta);
+	rate = (t_delta > 0.0) ? (double)async_sigs / t_delta : 0.0;
+	stress_metrics_set(args, 0, "SIGIO signals per sec", rate);
 
 finish:
 	/*  And ignore IO signals from now on */
@@ -228,8 +226,9 @@ stressor_info_t stress_sigio_info = {
 };
 #else
 stressor_info_t stress_sigio_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without fcntl() commands O_ASYNC, O_NONBLOCK, F_SETOWN, F_GETFL or F_SETFL"
 };
 #endif
