@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -357,10 +357,8 @@ die:
 	}
 	pr_dbg("%s: %" PRIu64 " messages sent\n", args->name, msgs);
 	dt = t2 - t1;
-	if (dt > 0.0) {
-		stress_misc_stats_set(args->misc_stats, 0,
-			"messages sent per sec", (double)msgs / dt);
-	}
+	if (dt > 0.0)
+		stress_metrics_set(args, 0, "messages sent per sec", (double)msgs / dt);
 
 	return rc;
 }
