@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -693,7 +693,7 @@ static void stress_stream_init_index(
 		idx[i] = i;
 
 	for (i = 0; i < n; i++) {
-		register const uint64_t j = stress_mwc64() % n;
+		register const uint64_t j = stress_mwc64modn(n);
 		register const uint64_t tmp = idx[i];
 
 		idx[i] = idx[j];
@@ -853,9 +853,9 @@ static int stress_stream(const stress_args_t *args)
 		pr_inf("%s: memory rate: %.2f MB read/sec, %.2f MB write/sec, %.2f Mflop/sec"
 			" (instance %" PRIu32 ")\n",
 			args->name, mb_rd_rate, mb_wr_rate, fp_rate, args->instance);
-		stress_misc_stats_set(args->misc_stats, 0, "memory read rate (MB per sec)", mb_rd_rate);
-		stress_misc_stats_set(args->misc_stats, 1, "memory write rate (MB per sec)", mb_wr_rate);
-		stress_misc_stats_set(args->misc_stats, 2, "memory rate (Mflop per sec)", fp_rate);
+		stress_metrics_set(args, 0, "memory read rate (MB per sec)", mb_rd_rate);
+		stress_metrics_set(args, 1, "memory write rate (MB per sec)", mb_wr_rate);
+		stress_metrics_set(args, 2, "memory rate (Mflop per sec)", fp_rate);
 	} else {
 		if (args->instance == 0)
 			pr_inf("%s: run duration too short to determine memory rate\n", args->name);
