@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,10 +33,10 @@ UNEXPECTED
 
 static const stress_help_t help[] = {
 	{ "s N","switch N",	 	"start N workers doing rapid context switches" },
-	{ NULL,	"switch-ops N",	 	"stop after N context switch bogo operations" },
 	{ NULL, "switch-freq N", 	"set frequency of context switches" },
 	{ NULL, "switch-method M",	"mq | pipe | sem-sysv" },
-	{ NULL, NULL, 		 NULL }
+	{ NULL,	"switch-ops N",	 	"stop after N context switch bogo operations" },
+	{ NULL, NULL, 			NULL }
 };
 
 typedef struct {
@@ -71,7 +71,7 @@ static void stress_switch_rate(
 	const double t_end,
 	uint64_t counter)
 {
-	pr_inf("%s: (%s) %.2f nanoseconds per context switch (based on parent run time)\n",
+	pr_inf("%s: (%s) %.2f nanosecs per context switch (based on parent run time)\n",
 		args->name, method,
 		((t_end - t_start) * STRESS_NANOSECOND) / (double)counter);
 }
@@ -137,6 +137,7 @@ static int stress_switch_pipe(
 	pid_t pid;
 	int pipefds[2];
 	size_t buf_size;
+	char *buf;
 
 	(void)memset(pipefds, 0, sizeof(pipefds));
 #if defined(HAVE_PIPE2) &&	\
@@ -161,6 +162,15 @@ static int stress_switch_pipe(
 	buf_size = args->page_size;
 #endif
 
+	buf = malloc(buf_size);
+	if (!buf) {
+		pr_fail("%s: pipe read/write buffer allocation failed\n",
+			args->name);
+		(void)close(pipefds[0]);
+		(void)close(pipefds[1]);
+		return EXIT_FAILURE;
+	}
+
 #if defined(F_SETPIPE_SZ)
 	if (fcntl(pipefds[0], F_SETPIPE_SZ, buf_size) < 0) {
 		pr_dbg("%s: could not force pipe size to 1 page, "
@@ -186,11 +196,9 @@ again:
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
+		free(buf);
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		char buf[buf_size];
-
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -199,7 +207,7 @@ again:
 		while (keep_stressing_flag()) {
 			ssize_t ret;
 
-			ret = read(pipefds[0], buf, sizeof(buf));
+			ret = read(pipefds[0], buf, buf_size);
 			if (ret <= 0) {
 				if (errno == 0)	/* ret == 0 case */
 					break;
@@ -215,13 +223,11 @@ again:
 		(void)close(pipefds[0]);
 		_exit(EXIT_SUCCESS);
 	} else {
-		char buf[buf_size];
 		int status;
 		double t_start;
 		uint64_t delay = switch_delay;
 
 		/* Parent */
-		(void)setpgid(pid, g_pgrp);
 		(void)close(pipefds[0]);
 		(void)memset(buf, '_', buf_size);
 
@@ -231,7 +237,7 @@ again:
 
 			inc_counter(args);
 
-			ret = write(pipefds[1], buf, sizeof(buf));
+			ret = write(pipefds[1], buf, buf_size);
 			if (ret <= 0) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					continue;
@@ -259,6 +265,8 @@ again:
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	free(buf);
+
 	return EXIT_SUCCESS;
 }
 
@@ -304,7 +312,6 @@ again:
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -333,8 +340,6 @@ again:
 		struct sembuf sem;
 
 		/* Parent */
-		(void)setpgid(pid, g_pgrp);
-
 		t_start = stress_time_now();
 		do {
 			inc_counter(args);
@@ -422,7 +427,6 @@ again:
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -438,13 +442,11 @@ again:
 		uint64_t delay = switch_delay;
 
 		/* Parent */
-		(void)setpgid(pid, g_pgrp);
-
 		t_start = stress_time_now();
 		do {
-			inc_counter(args);
 			unsigned int prio;
 
+			inc_counter(args);
 			if (mq_receive(mq, (char *)&msg, sizeof(msg), &prio) < 0)
 				break;
 
