@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2023 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -410,7 +410,7 @@ do_read:
 		counter = (double)get_counter(args);
 
 		rate = (counter > 0.0) ? duration / counter : 0.0;
-		stress_misc_stats_set(args->misc_stats, 0, "nanosecs per page fault", rate * 1000000000.0);
+		stress_metrics_set(args, 0, "nanosecs per page fault", rate * STRESS_DBL_NANOSECOND);
 
 		(void)memset(&wake, 0, sizeof(wake));
 		wake.start = (uintptr_t)data;
